#!/bin/bash
export PATH=$PATH:/bin:/usr/sbin:/usr/bin
if [[ "$(/usr/bin/id|cut -d"(" -f1|cut -d"=" -f2)" != "0" ]]; then
        echo "Please run this script as root."
        echo ""
        exit 1
fi
umask 0022
scriptDir=$(dirname $0)
cd $scriptDir
if [ "$scriptDir" = "." ]; then
	scriptDir=`pwd`
fi

AUTOMATIC=0
if [ "$1" = "-a" ]; then
	AUTOMATIC=1
fi

UNAME=`uname`
if [ "$UNAME" = "Linux" ]; then
        GREP=/bin/grep
        HTCONF="/var/opt/sun/xvm/uce/etc/uce_server/httpd.conf"
        if [ ! -f $HTCONF ]; then
                HTCONF="/etc/uce_server/httpd.conf"
        fi
else
        GREP=/usr/sfw/bin/ggrep
        HTCONF="/var/opt/sun/xvm/uce/etc.opt/server/uce_server/httpd.conf"
        if [ ! -f $HTCONF ]; then
                HTCONF="/etc/opt/SUNWuce/server/uce_server/httpd.conf"
        fi
fi

if [ ! -f $HTCONF ]; then
        echo "ERROR: Unable to find the location of the Ops Center httpd.conf. Make sure you run this on the Enterprise Controller."
        echo ""
        exit 1
fi
if [ ! -x $GREP ]; then
	echo "ERROR: Could not run $GREP"
	echo ""
	exit 1
fi

LOCAL_KB=`${GREP} -A10 "Listen 8005" $HTCONF | grep "^DocumentRoot" | head -1 | awk '{print $2}'`
if [ "$LOCAL_KB" = "" ]; then
	echo "ERROR: Could not obtain 'local-kb' folder from ${HTCONF}"
	echo ""
	echo "Did you remember to upload the KB and switch to disconnected mode in the UI first?"
	echo "Go to Administration -> Setup Connection Mode -> 'Load Bundle' and then 'Switch to Disconnected Mode'"
	echo ""
	echo "Run 'harvester.sh --instructions' for more information."
	echo ""
	exit 1
fi
### remove quotes 
LOCAL_KB=`echo $LOCAL_KB | sed -e 's/"//g'`

##### Do we have local-kb in there?
RIGHTFOLDER=`echo $LOCAL_KB | grep local-kb`
if [ "$RIGHTFOLDER" = "" ]; then
	### does it have public instead?
	PUBLICFOLDER=`echo $LOCAL_KB | grep public`
	if [ ! "$PUBLICFOLDER" = "" ]; then
		echo "ERROR: Your EC httpd.conf is not configured properly. This is a known issue in the current EC ver - BUG 7197429"
		echo "       If you apply the workaround below, you will need to re-apply it once the EC is restarted."
		echo ""
		echo "A temporary workaround (until next EC restart):"
		echo "1) Stop the uce-server service:"
		if [ "$UNAME" = "Linux" ]; then
			echo "# /etc/init.d/uce_server stop"

		else
			echo "# svcadm disable -t uce-server"
		fi
		echo
		echo "2) Edit $HTCONF, look for 'Listen 8005', and under it, change from:"
		echo "DocumentRoot $LOCAL_KB"
		echo " - to -"
		REPLACED=`echo $LOCAL_KB | sed -e 's/public/local-kb/'`
		echo "DocumentRoot $REPLACED"
		echo "(basically change at the end from 'public' to 'local-kb')"
		echo 
		echo "3) Start the uce-server service:"
		if [ "$UNAME" = "Linux" ]; then
			echo "# /etc/init.d/uce_server start"
		else
			echo "# svcadm enable uce-server"
		fi
		echo
		echo "Rerun this copy script after the workaround was applied. You may also need to re-upload the"
		echo "KB bundle through the HTML User Interface after applying the above workaround."
		echo
		exit 1
	else
		echo "ERROR: Your EC httpd.conf is not pointing to a local-kb folder. Please contact Oracle Support."
		echo ""
		echo "Got $LOCAL_KB from $HTCONF"
		echo ""
		exit 1
	fi
fi

### Make sure uce-server is back online in case user forgot after the change above
if [ "$UNAME" = "Linux" ]; then
	SERVICESTAT=`/etc/init.d/uce_server status |grep "is running"`
	if [ "$SERVICESTAT" = "" ]; then
		echo "ERROR: uce-server is not online, please make sure it is running. Try:"
		echo 
		echo "# /etc/init.d/uce_server start"
		echo ""
		exit 1
	fi
else
	SERVICESTAT=`svcs -H uce-server |awk '{print $1}'`
	if [ ! "$SERVICESTAT" = "online" ]; then
		echo "ERROR: uce-server is not online, please make sure it is running. Try:"
		echo 
		echo "# svcadm enable uce-server"
		echo
		exit 1
	fi
fi

echo ""
echo "OK: Found local-kb/all_unsigned folder here:"
echo "$LOCAL_KB/all_unsigned/"
mkdir -p ${LOCAL_KB}/all_unsigned

if [ -d ${LOCAL_KB}/all_unsigned ]; then
    chown root:root ${LOCAL_KB}/all_unsigned
    chmod 755 ${LOCAL_KB}/all_unsigned
    echo ""
    ### Do we have any zip files?
    LINES=`ls *.zip 2>/dev/null| grep -v OCDoctor-latest.zip 2>/dev/null | wc -l 2>/dev/null| awk '{print $1}' 2>/dev/null`
    if [ $LINES -ge 1 ]; then
    	if [ $AUTOMATIC -ne 1 ]; then
    		printf "Found $LINES ZIP files, do you want to copy them to the path above? (use -a to automate this) [Y/n] "
    		read yesno
    		if [ "$yesno" = "" ]; then
    			yesno="y"
    		fi
    		if [ "$yesno" = "yes" ]; then
    			yesno="y"
    		fi
    		if [ "$yesno" = "Y" ]; then
    			yesno="y"
    		fi
    		if [ "$yesno" != "y" ]; then
    			echo "Canceled."
    			echo
    			exit
    		fi
    	fi
    	echo "Copying $LINES ZIP files..."
    	ls *.zip 2>/dev/null| grep -v OCDoctor-latest.zip 2>/dev/null| while read file; do
    		if [ -f ${LOCAL_KB}/all_unsigned/$file ]; then		### if file exists, compare the two
    			diff $file ${LOCAL_KB}/all_unsigned/$file >/dev/null 2>/dev/null
    			if [ $? -eq 0 ]; then
    				echo $file already exists, skipping
    			else
    				echo "File exists but it is different. Copying new file $file"
   					/bin/cp -f $file ${LOCAL_KB}/all_unsigned/
   				fi
   			else
   				echo Copying $file
   				cp $file ${LOCAL_KB}/all_unsigned/
    		fi
    	done
    	echo ""
    	echo "Setting proper permissions..."
    	chown -R root:root ${LOCAL_KB}/all_unsigned
    	chmod -R 755 ${LOCAL_KB}/all_unsigned
    	echo ""
    	
    else
    	echo "No patches were found in current folder ($scriptDir), not copying."
    	echo ""
    fi
    if [ -f OCDoctor-latest.zip ]; then
    	if [ $AUTOMATIC -ne 1 ]; then
    		echo ""
    		printf "Update the OCDoctor version on the EC? (use -a to automate this) [Y/n] "
    		read yesno
    		if [ "$yesno" = "" ]; then
    			yesno="y"
    		fi
    		if [ "$yesno" = "yes" ]; then
    			yesno="y"
    		fi
    		if [ "$yesno" = "Y" ]; then
    			yesno="y"
    		fi
    		if [ "$yesno" != "y" ]; then
    			echo "Canceled."
    			echo
    			exit
    		fi
    	fi
    	echo "Updating OCDoctor in /var/opt/sun/xvm/OCDoctor/"
    	echo
    	PWD=`pwd`
    	/var/opt/sun/xvm/OCDoctor/OCDoctor.sh --ui --update --file $PWD/OCDoctor-latest.zip
    fi
else
	echo "ERROR: Could not create ${LOCAL_KB}/all_unsigned"
	echo ""
	exit 1
fi
echo
echo "All done."
echo ""
