#!/bin/bash
D_NAME=$1				# Name of the download
FILE_URL=$2				# updates.oracle.com
DEST_FOLDER=$3			# /var/tmp/...
SIG_URL=$4 				# updates.oracle.com...sig
SIGNATURES_FILE=$5		# This may be empty

if [ "$SIGNATURES_FILE" = "" ]; then
	SIGNATURES_FILE="signatures_file_not_defined"
fi

if [ "$D_NAME" = "" ] || [ "$FILE_URL" = "" ] || [ "$DEST_FOLDER" = "" ]; then
	echo "Usage: download.sh <Name of download> <URL> <Destination folder> [Signature link]"
	echo
	exit 1
fi

umask 0022

COOKIES_FILE="/tmp/harvester-download-cookies.txt"
WGETCOOKIES_LOCK_FILE="/tmp/harvester-download-cookies.lock"
sync

##############################
DEBUG=false				### Debug for download.sh
ERRORDEBUG=true			### only print how download.sh was executed, if exit code is 1 (so you can try and run it yourself from command line
##############################

export PATH=$PATH:/usr/sbin:/usr/bin
scriptDir=$(dirname $0)
cd $scriptDir


UNAME=`uname`
if [ $UNAME = "Linux" ];then
	MD5SUM="/usr/bin/md5sum"
    MD5SUM_ARGS=""
else
	MD5SUM="/bin/digest"
	MD5SUM_ARGS="-a md5"
fi
    
check_signature_against_signature_array(){
	SIGARRAY=$1				#### file with all signatures
	FILE=$2					#### File to verify signature
	SHORTFILE=`basename $FILE`
	if [ ! -f $SIGARRAY ]; then
		echo "ERROR: Cannot find signature array file $SIGARRAY"
		return 1
	fi
	ISFIRMWARE=`echo $SIGARRAY | grep signatures-firmware`
	ISSOLARIS=`echo $SIGARRAY | grep signatures-solaris`
	if [ ! "$ISFIRMWARE" = "" ]; then
		### It is firmware
		SIGNATUREFROMARRAY=`grep "^$SHORTFILE|" $SIGARRAY | awk -F"|" '{print $2}'`
		#echo "grep \"^$SHORTFILE|\" $SIGARRAY "
		SIGNATUREFROMFILE=`$MD5SUM $MD5SUM_ARGS $FILE | awk '{print $1}'`
		echo "check_signature_against_signature_array (FIRMWARE): FILE:$FILE    SIGNATUREFROMFILE:$SIGNATUREFROMFILE   SIGARRAY:$SIGARRAY  SIGNATUREFROMARRAY:$SIGNATUREFROMARRAY" >> ${WGET_LOG}
		if [ "$SIGNATUREFROMFILE" = "$SIGNATUREFROMARRAY" ] && [ ! "$SIGNATUREFROMARRAY" = "" ]; then
			#echo "OK: Signatures match"
			return 0
		else 
			#echo "NOT OK: Signatures DO NOT match"
			return 1
		fi
	elif [ ! "$ISSOLARIS" = "" ]; then
		### Solaris
		SIGNATUREFROMARRAY=`grep "^$SHORTFILE|" $SIGARRAY | awk -F"|" '{print $2}'`
		SIGNATUREFROMFILE=`$scriptDir/get_patch_finger_print.sh -blob_file_name $FILE | head -1`
		if [ "$SIGNATUREFROMFILE" = "<Response_List>" ]; then
			echo "ERROR: Unable to obtain fingerprint using the following command:"
			echo "# $scriptDir/get_patch_finger_print.sh -blob_file_name $FILE"
			echo "OUTPUT RECEIVED:"
			OUTPUT=`$scriptDir/get_patch_finger_print.sh -blob_file_name $FILE`
			printf "$OUTPUT\n"
			
			echo "ERROR: Unable to obtain fingerprint using the following command:" >> ${WGET_LOG}
			echo "# $scriptDir/get_patch_finger_print.sh -blob_file_name $FILE" >> ${WGET_LOG}
			echo "OUTPUT RECEIVED:" >> ${WGET_LOG}
			printf "$OUTPUT\n" >> ${WGET_LOG}
			
			return 1
		fi
		echo "check_signature_against_signature_array (SOLARIS): FILE:$FILE    SIGNATUREFROMFILE:$SIGNATUREFROMFILE   SIGARRAY:$SIGARRAY   SIGNATUREFROMARRAY:$SIGNATUREFROMARRAY" >> ${WGET_LOG}
		if [ "$SIGNATUREFROMFILE" = "$SIGNATUREFROMARRAY" ] && [ ! "$SIGNATUREFROMARRAY" = "" ]; then
			#echo "OK: Signatures match SIGNATUREFROMARRAY:$SIGNATUREFROMARRAY  SIGNATUREFROMFILE:$SIGNATUREFROMFILE"
			return 0
		else 
			#echo "NOT OK: Signatures DO NOT match  SIGNATUREFROMARRAY:$SIGNATUREFROMARRAY  SIGNATUREFROMFILE:$SIGNATUREFROMFILE"
			return 1
		fi
	else
		echo "ERROR: Uknown signature type $FILE  SIGARRAY:$SIGARRAY  SHORTFILE:$SHORTFILE"  >> ${WGET_LOG}
		echo "ERROR: Uknown signature type $FILE"
		return 1
	fi
}




############### Read config #################
export TEMP_CONFIG_VARS_FILE="/var/tmp/harvester-vars.$$"
if [ -f ./config.ini ]; then
	rm -f $TEMP_CONFIG_VARS_FILE >/dev/null 2>/dev/null		# cleanup just in case
	cat ./config.ini | grep -v "^#" | while read line; do
		if [ ! "$line" = "" ]; then
			PARAM=`echo "$line" | cut -d'=' -f1`
			VAL=`echo "$line" | cut -d'=' -f2- | sed -e 's/\"//g'`		### remove " from it
			if [ "$PARAM" = "HTTP_PASSWD" ]; then		### minor correction to make config.ini more user friendly
				PARAM="HTTP_PASSWD_UNESCAPED"
				if [ -f "$VAL" ]; then			##### Allow specifying a filename instead of password
					read -r VAL < ${VAL}
				fi
			fi
			if [ "$PARAM" = "PROXY_PASSWD" ]; then		### minor correction to make config.ini more user friendly
				PARAM="PROXY_PASSWD_UNESCAPED"
				if [ -f "$VAL" ]; then			##### Allow specifying a filename instead of password
					read -r VAL < ${VAL}
				fi
			fi
			ENVPARAM=`eval "echo \\$${PARAM}"`              ### the current value of this parameter
			#echo "PARAM:$PARAM VAL:$VAL ENVPARAM:$ENVPARAM"
			if [ "$ENVPARAM" = "" ]; then           # if not set already
				echo "export $PARAM=$VAL" >> $TEMP_CONFIG_VARS_FILE
			fi
		fi
	done
fi
if [ -f $TEMP_CONFIG_VARS_FILE ]; then
	### Need this since bash creates a subshell for the while loop and 'forgets' the variables
	source $TEMP_CONFIG_VARS_FILE
	rm -f $TEMP_CONFIG_VARS_FILE
fi
############### End of Read config #################


URL_FILE="/var/tmp/harvester_url.$$"
#HTTP_USER_AGENT="Aduva agent"
if [ "$HTTP_USER_AGENT" = "" ]; then		### we may have a different one from config.ini, so don't overwrite it
	HTTP_USER_AGENT="Wget/1.10.2"			
fi
UNAME=`uname`
if [ $UNAME = "Linux" ];then
	WGET="/usr/bin/wget --timeout=60 --tries 5 --retry-connrefused  --secure-protocol=TLSv1_2 -v"
	### The TLS issue is explained best in this thread: http://lists.opencsw.org/pipermail/bug-notifications/2013-April/011703.html
	### Basically, there is a problem with updates.oracle.com SSL implementation - it dies if it gets a cipher string longer than 256b
	WGETPRINT="/usr/bin/wget"
	OPENSSL=/usr/bin/openssl
	EPOCH=`date +%s`
	DF="df -Pk"
else
	WGET="/usr/sfw/bin/wget --timeout=60 --tries 5 --retry-connrefused  --secure-protocol=TLSv1_2 -v"
	### The TLS issue is explained best in this thread: http://lists.opencsw.org/pipermail/bug-notifications/2013-April/011703.html
	### Basically, there is a problem with updates.oracle.com SSL implementation - it dies if it gets a cipher string longer than 256b
	WGETPRINT="/usr/sfw/bin/wget"
	OPENSSL=/usr/sfw/bin/openssl
	EPOCH=`nawk 'BEGIN{print srand()}'`
	DF="df -k"
fi



##################### Read username/password #############
# preserve leading and trailing spaces if any. Don't treat \ as special char
if [ ! "$PASSWORD_FILE" = "" ]; then			# if password file param was mentioned
	read -r HTTP_PASSWD_UNESCAPED < ${PASSWORD_FILE}
fi
# escape double quote
export HTTP_PASSWD=`echo $HTTP_PASSWD_UNESCAPED | sed s/\"/\\\\\\\\\"/g`		### Needed by download.sh

###### indicate in the log if the operation was using certs
echo "certFlag is " $CERTFLAG >> ${WGET_LOG}

################### Deal with proxies ########################
if [ "${PROXY_PASSWORD_FILE}" != "" ]; then
   read -r PROXY_PASSWD_UNESCAPED < ${PROXY_PASSWORD_FILE}
fi
if [ ! "$PROXY_PASSWD_UNESCAPED" = "" ]; then			### it can be set either by a password file (above) or through config.ini
	PROXY_PASSWD=`echo $PROXY_PASSWD_UNESCAPED | sed s/\"/\\\\\\\\\"/g`
fi

PROXY_STRING=""
if [ ! "${PROXY_URL}" = "" ];then
	#echo "Using http/https proxy $PROXY_URL"
	PROXY_STRING=${PROXY_STRING}" --proxy -e https_proxy=${PROXY_URL} -e http_proxy=${PROXY_URL} "   
	if [ "${PROXY_USER}" != "" ] && [ "${PROXY_PASSWD}" != "" ]; then 
		#echo "Using http/https proxy user $PROXY_USER"
   		PROXY_STRING=${PROXY_STRING}" --proxy-user=${PROXY_USER} --proxy-passwd=${PROXY_PASSWD}"
	fi
fi


wget_wrapper(){
	BAD_FILE_RETRIES=5		#### How many times to rerun wget in case of an error
	SLEEP_BETWEEN_RETRIES=2 #### How many seconds to wait between each wget command
	while [ $BAD_FILE_RETRIES -gt 0 ]; do
		BAD_FILE_RETRIES=`expr $BAD_FILE_RETRIES - 1` 
	    #encode special chars %,@,/,: in HTTP_USER and HTTP_PASSWD
	    HTTP_USER_ENCODED1=`echo $HTTP_USER | sed 's/:/%3A/g;s/@/%40/g;s/\//%2f/g;s/\*/%2a/g;s/+/%2b/g;s/,/%2c/g;s/;/%3b/g;s/=/%3d/g;s/!/%21/g;s/#/%23/g;s/\&/%26/g;s/?/%3f/g;s/(/%28/g;s/)/%29/g;s/\[/%5b/g;s/\]/%5d/g'`
	    # escaping ' is a special case
	    HTTP_USER_ENCODED=`echo $HTTP_USER_ENCODED1 | sed s/\'/%27/g`
	    HTTP_PASSWD_ENCODED1=`echo $HTTP_PASSWD | sed 's/:/%3A/g;s/@/%40/g;s/\//%2f/g;s/\*/%2a/g;s/+/%2b/g;s/,/%2c/g;s/;/%3b/g;s/=/%3d/g;s/!/%21/g;s/#/%23/g;s/\&/%26/g;s/?/%3f/g;s/(/%28/g;s/)/%29/g;s/\[/%5b/g;s/\]/%5d/g'`
	    # escaping ' is a special case
	    HTTP_PASSWD_ENCODED=`echo $HTTP_PASSWD_ENCODED1 | sed s/\'/%27/g`
	
		if [ "$HTTP_USER_ENCODED" = "" ]; then			#### happens if user/pass not defined and trying to check updates
			URL="https://$3"
		else
			URL="https://$HTTP_USER_ENCODED:$HTTP_PASSWD_ENCODED@$3"
	    fi
	    echo "$URL" > ${URL_FILE}
	    chmod 700 ${URL_FILE}
	    
		WGET_OUTPUT=`${WGET} --load-cookies=${COOKIES_FILE} --keep-session-cookies  "--user-agent=${HTTP_USER_AGENT}" ${PROXY_STRING} $CERTFLAG -i ${URL_FILE} -O $2 2>&1`
		RETVAL=$?
		
		#### Did we get certification error?
		DID_CERT_FAIL=`printf "%s\n" "$WGET_OUTPUT" | grep -i "Certificate verification error for updates.oracle.com" | grep -i "^ERROR" 2>/dev/null`
		if [ ! "$DID_CERT_FAIL" = "" ]; then
			echo ""
			echo "[ERROR] SSL Certificate error. Please review config.ini for a new 'CERTFLAG' parameter and activate it if needed"
			echo ""
			rm -f ${URL_FILE}        
			return 1
		fi
			
		#### Need to check if we got AUTH failed, if so, we will have to launch the login page method
		DID_AUTH_FAIL=`printf "%s\n" "$WGET_OUTPUT" | grep -i " 401 " 2>/dev/null`
		if [ ! "$DID_AUTH_FAIL" = "" ]; then       # we got 401, meaning, failed
			echo "Thread $$ was redirected to a login page, following and saving coookies" >> ${WGET_LOG}
			### Need to follow the login page, but first, must lock the cookie file
			set -o noclobber			### enable
			while ! `echo "$$" 2>/dev/null > "$WGETCOOKIES_LOCK_FILE" ` ; do					#### will fail if exists
				### Check if its a valid lock
				LOCKPROC=`cat $WGETCOOKIES_LOCK_FILE`
				ps -p $LOCKPROC >/dev/null 2>/dev/null
				if [ $? -ne 0 ]; then
					echo "[LOCK] Lock file found at $WGETCOOKIES_LOCK_FILE but the process ID in it ($LOCKPROC) is no longer alive, removing lock file" >> ${WGET_LOG}
					echo "[LOCK] Lock file found at $WGETCOOKIES_LOCK_FILE but the process ID in it ($LOCKPROC) is no longer alive, removing lock file"
					rm -f $WGETCOOKIES_LOCK_FILE
				else
					echo "[LOCK] Waiting 10 seconds for lock on $WGETCOOKIES_LOCK_FILE in thread $$ (delete this file if this is incorrect)" >> ${WGET_LOG}
					sleep 10		### Didn't get lock, sleep
				fi
			done
			set +o noclobber			### disable
			#### Got lock here
			echo "[OK] Got cookie file lock in thread $$" >> ${WGET_LOG}
			#### Need to save cookies now while going to the LOGIN page
			#### Extract login link
			### extract the URL it is redirected to - only the "login....." part only, without https:// etc
			WGET_REDICRECT_URL=`printf "%s\n" "$WGET_OUTPUT" | grep -i "^Location" | awk '{print $2}' 2>/dev/null | sed -e 's/https:\/\///' 2>/dev/null`
			if [ "$WGET_REDICRECT_URL" = "" ]; then
				### in case we were not able to extract the new URL, use the same one like originally provided, again (should not get here unless something went wrong)
				echo "$URL" > ${URL_FILE}
			else
				### If we have a new redirect login page, use that
				if [ "$HTTP_USER_ENCODED" = "" ]; then			### won't have username in testing, accomodate that to prevent error
					echo "https://$WGET_REDICRECT_URL" > ${URL_FILE}
				else
					echo "https://$HTTP_USER_ENCODED:$HTTP_PASSWD_ENCODED@$WGET_REDICRECT_URL" > ${URL_FILE}
		    	fi
		    	chmod 700 ${URL_FILE}
			fi
			#### Now download the real file - going to the login page will actually redirect to the final file
			WGET_OUTPUT=`${WGET} --save-cookies=${COOKIES_FILE} --keep-session-cookies "--user-agent=${HTTP_USER_AGENT}" -a ${WGET_LOG} ${PROXY_STRING} $CERTFLAG -i ${URL_FILE} -O $2`
			RETVAL=$?
			rm -f $WGETCOOKIES_LOCK_FILE >/dev/null 2>/dev/null		### clear lock
		else
			### Auth did not fail (meaning, it used the cookies) , put the output in the log as it's a valid output to review
			printf "%s\n" "$WGET_OUTPUT" >> ${WGET_LOG}
		fi
			
		rm -f ${URL_FILE}		### clean URL file (has a password in it!)
		
	    if [ $RETVAL -ne 0 ]; then
	        $DEBUG && echo "Error downloading $3, retrying..."
	        echo "Error downloading $3 (return code $RETVAL), retrying..." >> ${WGET_LOG}
	        # Continue with the loop again
		else
			if [ -s $2 ]; then			#### file exists and size greater than 0
				#### Check if it's ZIP, if so check with ZIP
				echo "$3" | grep ".zip$" >/dev/null 2>/dev/null
				if [ $? -ne 0 ]; then		#### it's NOT a ZIP file
					echo "[OK] Downloaded $3" >> ${WGET_LOG}
					BAD_FILE_RETRIES=0
					return 0
				else		
					#### Test ZIP file
					sync
					unzip -t $2 >/dev/null 2>/dev/null
					if [ $? -ne 0 ]; then
						echo "Error downloading $3 (bad ZIP file), retrying in a few seconds..." >> ${WGET_LOG}
						$DEBUG && echo "Error downloading $3 (bad ZIP file), retrying in a few seconds..."
	        			rm -f ${URL_FILE}
	        		else
	        			echo "[OK] Downloaded ZIP file $3" >> ${WGET_LOG}
						BAD_FILE_RETRIES=0
						return 0
	        		fi
				fi
			else
				if [ -f $2 ]; then
					rm -f $2
				fi
				echo "Error downloading $3 (got empty file), retrying in a few seconds..." >> ${WGET_LOG}
				$DEBUG && echo "Error downloading $3 (got empty file), retrying in a few seconds..."
	        	rm -f ${URL_FILE}
			fi
	    fi
	    sleep $SLEEP_BETWEEN_RETRIES;			#### sleep before the next retry
	done
	### We got here only if download failed
	$DEBUG && echo "Error downloading $3, Please check ${WGET_LOG} for more information"
	echo "ERROR: Final Error downloading $3, could not download" >> ${WGET_LOG}
	rm -f ${URL_FILE}        
	return 1
}

check_sig_against_specific_sig_file() {
	FULL_FILE_PATH=$1
	FULL_SIG_PATH=$2
	#FULL_OK_PATH="${FULL_SIG_PATH}.OK"
	#if [ -f $FULL_OK_PATH ]; then
	#	$DEBUG && echo "Signature was already checked"
	#	return 0
	#else
		$OPENSSL dgst -binary -sha1 $FULL_FILE_PATH > ${FULL_FILE_PATH}.generated
		diff ${FULL_FILE_PATH}.generated $FULL_SIG_PATH >/dev/null 2>/dev/null
		#echo "${FULL_FILE_PATH}.generated $FULL_SIG_PATH"
		#read
   		RC=$?
   		rm -f ${FULL_FILE_PATH}.generated
   		if [ ${RC} -ne 0 ]; then
   			$DEBUG && echo "check_sig_against_specific_sig_file(): Sig not the same  FULL_FILE_PATH:$FULL_FILE_PATH  FULL_SIG_PATH:$FULL_SIG_PATH"
   			return 1
   		else
   			$DEBUG && echo "check_sig_against_specific_sig_file(): Sig the same  FULL_FILE_PATH:$FULL_FILE_PATH  FULL_SIG_PATH:$FULL_SIG_PATH"
   			#touch $FULL_OK_PATH
   			return 0
		fi
	#fi
}

check_exclude(){
	FULL_URL=$1
	grep "$FULL_URL" exclude-list.txt >/dev/null 2>/dev/null
	if [ $? -eq 0 ]; then
		##### it's in the exclude list, skip
		$DEBUG && echo "EXCLUDED: $FULL_URL"
		sync
		exit 0
	fi
}

#### First check if we have the file downloaded already?
DEST_FILE=`basename "$FILE_URL"`
DEST_FILE_FULLPATH="${DEST_FOLDER}/${DEST_FILE}"
if [ ! "$SIG_URL" = "" ]; then
	SIG_FILE=`basename "$SIG_URL"`
	SIG_FILE_FULLPATH="${DEST_FOLDER}/${SIG_FILE}"
	#SIG_FILE_FULLPATH_OK="${SIG_FILE_FULLPATH}.OK"
fi
if [ -f $DEST_FILE_FULLPATH ]; then					#### IF file already exists
	$DEBUG && echo "File exists already ($DEST_FILE_FULLPATH)"
	#### Do we have sig file available?
	if [ "$SIG_URL" = "" ]; then
		##### Check signature if available
		if [ -f $SIGNATURES_FILE ]; then
			check_signature_against_signature_array $SIGNATURES_FILE $DEST_FILE_FULLPATH
		else
			NO_SIGNATURE_FILE=1		##### don't check for signature if no sig file array provided - this variable is just for explaining, not used
		fi
		SIGOK=$?
		$DEBUG && echo "No need to download main file as it already exists"
		#if [ -f ${DEST_FILE_FULLPATH}.OK ]; then	
		if [ $SIGOK -eq 0 ]; then
			echo "[${D_NAME}] Already exists"
			sync
			exit 0
		else
			echo "[${D_NAME}] Incorrect signature, redownloading"
			touch ${DEST_FILE_FULLPATH}.NEWSIG
			#$DEBUG && echo "File already exists but no OK file yet, checking it then"
			$DEBUG && echo "File already exists but bad signature, redownloading it then"
			sync
			$DEBUG && echo "Bad ZIP, removing"
			if [ -f $DEST_FILE_FULLPATH ]; then
				rm -f $DEST_FILE_FULLPATH >/dev/null 2>/dev/null
			fi
			rm -f $DEST_FILE_FULLPATH.moved_for_download >/dev/null 2>/dev/null		### not always there
		fi
	else
		#if [ ! -f $SIG_FILE_FULLPATH ]; then				#### IF SIG file doesn't exists, get it
		if [ -f $SIG_FILE_FULLPATH ]; then				#### Always get fresh signature file
			rm -f $SIG_FILE_FULLPATH
		fi
			$DEBUG && echo "Sig file does NOT exist ($SIG_FILE_FULLPATH)"
			wget_wrapper -O ${SIG_FILE_FULLPATH} ${SIG_URL}
			if [ $? -eq 0 ]; then
				$DEBUG && echo "OK - sig downloaded ${SIG_FILE_FULLPATH}"
			else
				$DEBUG && echo "ERROR downloading sig file ${SIG_URL}"
				if [ -f $DEST_FILE_FULLPATH ]; then
					rm -f $DEST_FILE_FULLPATH >/dev/null 2>/dev/null
				fi
				if [ -f $SIG_FILE_FULLPATH ]; then
					rm -f $SIG_FILE_FULLPATH >/dev/null 2>/dev/null
				fi
				rm -f $DEST_FILE_FULLPATH.moved_for_download >/dev/null 2>/dev/null		### not always there
				echo "[${D_NAME}] ERROR downloading sig file"
				$ERRORDEBUG && echo "./download.sh \"$D_NAME\" $FILE_URL $DEST_FOLDER \"$SIG_URL\" $SIGNATURES_FILE"
				sync
				exit 1
				
			fi
		#fi
		check_sig_against_specific_sig_file $DEST_FILE_FULLPATH $SIG_FILE_FULLPATH
		if [ $? -ne 0 ]; then
			$DEBUG && echo "check_sig_against_specific_sig_file returned error, will force re-download of both sig and file"
			rm -f $DEST_FILE_FULLPATH $SIG_FILE_FULLPATH >/dev/null 2>/dev/null
			rm -f $DEST_FILE_FULLPATH.moved_for_download >/dev/null 2>/dev/null		### not always there
			### Need to remove both sig file and file
		else
			$DEBUG && echo "check_sig_against_specific_sig_file returned OK on $DEST_FILE_FULLPATH"
			echo "[${D_NAME}] Already exists"
			sync
			exit 0
		fi
	fi			#### End of "do we have sig URL?	
fi

#if [ -f $SIG_FILE_FULLPATH_OK ]; then
	#rm -f $SIG_FILE_FULLPATH_OK >/dev/null 2>/dev/null		### Remove OK file as we're downloading fresh copy of both file and sig
#fi
rm -f $DEST_FILE_FULLPATH.moved_for_download >/dev/null 2>/dev/null		### not always there

#####
check_exclude ${FILE_URL}

######## Check for enough disk space #########
AVAILABLE=`$DF $DEST_FOLDER | tail -1 | awk '{print \$4}'`
REQUIRED=1500000		### this is 1.5G. When 8 threads running in parallel, some patches are can be as big as 600MB
if [ $AVAILABLE -lt $REQUIRED ]; then
        echo "ERROR [$D_NAME]: Not enough disk space on $DEST_FOLDER - Need ${REQUIRED}K but only have ${AVAILABLE}K" >> ${WGET_LOG}
        echo "ERROR [$D_NAME]: Not enough disk space on $DEST_FOLDER - Need ${REQUIRED}K but only have ${AVAILABLE}K"
        exit 1
fi
######## End of Check for enough disk space #########


##### Start downloading
$DEBUG && echo "Need to download"
wget_wrapper -O ${DEST_FILE_FULLPATH} ${FILE_URL}
if [ $? -eq 0 ]; then
	$DEBUG && echo "OK - file downloaded"
	if [ ! "$SIG_URL" = "" ]; then		#### Do we have sig link provided?
		if [ -f $SIG_FILE_FULLPATH ]; then
			$DEBUG && echo "Removing old sig file"
			rm -f $SIG_FILE						#### Remove old sig file if exists
		fi
		wget_wrapper -O ${SIG_FILE_FULLPATH} ${SIG_URL}
		if [ $? -eq 0 ]; then
			$DEBUG && echo "OK - sig downloaded"
		else
			echo "[${D_NAME}] ERROR downloading sig file"
			rm -f $DEST_FILE_FULLPATH $SIG_FILE_FULLPATH >/dev/null 2>/dev/null
			rm -f $DEST_FILE_FULLPATH.moved_for_download >/dev/null 2>/dev/null		### not always there
			$ERRORDEBUG && echo "./download.sh \"$D_NAME\" $FILE_URL $DEST_FOLDER \"$SIG_URL\" $SIGNATURES_FILE"
			sync
			exit 1
		fi
		check_sig_against_specific_sig_file $DEST_FILE_FULLPATH $SIG_FILE_FULLPATH
		if [ $? -ne 0 ]; then
			$DEBUG && echo "ERROR downloading ${FILE_URL} - signature mismatch"
			echo "[${D_NAME}] ERROR downloading, signatures do not match"
			rm -f $DEST_FILE_FULLPATH $SIG_FILE_FULLPATH >/dev/null 2>/dev/null
			rm -f $DEST_FILE_FULLPATH.moved_for_download >/dev/null 2>/dev/null		### not always there
			$ERRORDEBUG && echo "./download.sh \"$D_NAME\" $FILE_URL $DEST_FOLDER \"$SIG_URL\" $SIGNATURES_FILE"
			sync
			exit 1
		else
			$DEBUG && echo "OK - signature matches"
			echo "[${D_NAME}] Downloaded successfully"
			sync
			exit 0
		fi
	else
		if [ -f $SIGNATURES_FILE ]; then
			check_signature_against_signature_array $SIGNATURES_FILE $DEST_FILE_FULLPATH
			SIGOK=$?
			if [ $SIGOK -ne 0 ]; then
				$DEBUG && echo "ERROR downloading ${FILE_URL} - signature mismatch (fingerprint comparison)"
				echo "[${D_NAME}] ERROR downloading, signatures (fingerprints) do not match"
				#if [ -f $DEST_FILE_FULLPATH ]; then
				#	Keep the file, don't remove - for debugging and in case customer wants to copy the bad file (in case of KB error)
				#	rm -f $DEST_FILE_FULLPATH >/dev/null 2>/dev/null
				#fi
				rm -f $DEST_FILE_FULLPATH.moved_for_download >/dev/null 2>/dev/null		### not always there
				$ERRORDEBUG && echo "./download.sh \"$D_NAME\" $FILE_URL $DEST_FOLDER \"$SIG_URL\" $SIGNATURES_FILE"
				sync
				exit 1
			else
				$DEBUG && echo "OK: signature matches"
			fi
		else
			NO_SIGNATURE_FILE=1		##### don't check for signature if no sig file array provided - this variable is just for explaining, not used
		fi
		echo "[${D_NAME}] Downloaded successfully"
		sync
		exit 0

		
	fi
else
	if [ $ERRORDEBUG ]; then
		echo "[${D_NAME}] ERROR - Could not download. Command used:"
	else
		echo "[${D_NAME}] ERROR - Could not download"
	fi
	rm -f $DEST_FILE_FULLPATH $SIG_FILE_FULLPATH >/dev/null 2>/dev/null
	rm -f $DEST_FILE_FULLPATH.moved_for_download >/dev/null 2>/dev/null		### not always there
	$ERRORDEBUG && echo "./download.sh \"$D_NAME\" $FILE_URL $DEST_FOLDER \"$SIG_URL\" $SIGNATURES_FILE"
	sync
	exit 1
fi
sync
