#!/bin/sh
#
# Copyright 2006 Sun Microsystems, Inc.  All rights reserved.
# Use is subject to license terms.
#

#Required tools
if [ `uname` = "SunOS" ] ; then
    UCE_MAGIC="/var/opt/sun/xvm/uce/etc.opt/server/uce_magic"
    TAR="/usr/sfw/bin/gtar"
    UNZIP="/bin/unzip"
    ZIPINFO="/bin/zipinfo"
    FILE="/bin/file"
    #FILE_ARGS="-m $UCE_MAGIC"
    FILE_ARGS=""
    UNCOMPRES="/bin/uncompress"
    MD5SUM="/bin/digest"
    MD5SUM_ARGS="-a md5"
    EGREP="/usr/xpg4/bin/egrep"
	COMPRESSED_FILE_TYPE="compressed data block"
    TAR_FILE_TYPE="tar archive"
	ZIP_FILE_TYPE="ZIP archive"
else
    TAR="/bin/tar"
    UNZIP="/usr/bin/unzip"
    ZIPINFO="/usr/bin/zipinfo"
    FILE="/usr/bin/file"
    FILE_ARGS="-i"
    UNCOMPRES="/bin/gunzip"
    MD5SUM="/usr/bin/md5sum"
    MD5SUM_ARGS=""
    EGREP="/bin/egrep"
    COMPRESSED_FILE_TYPE="application/x-compress|application/compress"
    TAR_FILE_TYPE="application/x-tar"
    ZIP_FILE_TYPE="application/zip|application/x-zip"
fi

#Patch regex
PATCH_REGEX="[1-9][0-9]{5}-[0-9]{2}"
PATCH_SED_REGEX="[1-9][0-9]\{5\}-[0-9]\{2\}"

#Relevant files for finger_print
RELEVANT_FILES_REGEX="^($PATCH_REGEX)/(patchinfo|.+/pkgmap$|.+/pkginfo$)"
README_REGEX="^($PATCH_REGEX)/README.$PATCH_REGEX\$"

#Dependencies regexes
REQUIRES_REGEX='^Patches required.*:|^Required Patches.*:'
OBSOLETES_REGEX='^Patches accumulated and obsoleted by this patch:'
OBSOLETED_REGEX='^Obsolete.*by:'
CONFLICTS_REGEX='^Patches which conflict with this patch:'
OS_RELEASE_REGEX='^Sun.*OS Release:'

#Patch regex
TMP_WORK_DIR="/tmp/.get_patch_fp"$$
README_TMP_WORK_FILE="$TMP_WORK_DIR/.README"
TMP_MD5_FILE="$TMP_WORK_DIR/.MD5"

USAGE="Usage: $0 -blob_file_name BLOB_PATH"

#Valid exit status
OS_S_OK=0

trap Finalize INT QUIT KILL TERM EXIT

Response()
{
	ERR=$1
	MSG=$2
	VAL=$3
	echo "<Response_List>
	<Aduva_Response>
		<ERRCODE>${ERR}</ERRCODE>
		<ERR_MSG><![CDATA[${MSG}]]></ERR_MSG>
		<RESPONSE_KEYVALUE>
			<PARAM>fingerprint</PARAM>
			<VAL><![CDATA[${VAL}]]></VAL>
		</RESPONSE_KEYVALUE>
	</Aduva_Response>
</Response_List>"
	exit 1		#### the error codes are not defined
        if [ $ERR -gt 0 ] ; then
           exit 1
        fi
	   
	exit $ERR
}

FirstAssert()
{

        if [ ! -x $TAR ] ; then
	   Response E_DTT_TAR_NOT_FOUND "$TAR not installed"
        fi

        if [ ! -x $UNZIP ] ; then
	   Response E_DTT_UNZIP_NOT_FOUND "$UNZIP not installed"
        fi

        if [ ! -x $FILE ] ; then
	   Response E_DTT_FILE_NOT_FOUND "$FILE not installed"
        fi

        if [ ! -x $UNCOMPRES ] ; then
	   Response E_DTT_UNCOMPRES_NOT_FOUND "$UNCOMPRES not installed"
        fi

        if [ ! -x $MD5SUM ] ; then
	   Response E_DTT_MD5SUM_NOT_FOUND "$MD5SUM not installed"
        fi
	
}

Identify()
{
	FILE_TYPE=`$FILE $FILE_ARGS $BLOB_FILE_NAME 2>/dev/null | cut -f2 -d':'`
    if [ $? -ne 0 ];then
        Response E_DTT_FILE_CMD_FAILED "Can't identify $BLOB_FILE_NAME"
    fi
	for next_type in "$COMPRESSED_FILE_TYPE" "$TAR_FILE_TYPE" "$ZIP_FILE_TYPE"
	do
		echo "$FILE_TYPE" | $EGREP -i "$next_type" > /dev/null 2>&1
		RES=$?
		if [ $RES -eq 0 ];then
			FILE_TYPE=$next_type
			return 0;
		fi
	done
		
   Response E_DTT_UNKNOWN_BLOB_FORMAT "Invalid file-type:$FILE_TYPE"
}	 

HandleBlob()
{
	if [ "$FILE_TYPE" = "$COMPRESSED_FILE_TYPE" ];then
		LIST_CMD="$TAR tZ -f $BLOB_FILE_NAME"
		EXTRACT_CMD="$TAR xZ -f $BLOB_FILE_NAME -C $TMP_WORK_DIR"
		EXTRACT_STDOUT_CMD="$TAR xZ -f $BLOB_FILE_NAME --to-stdout"
	elif [ "$FILE_TYPE" = "$TAR_FILE_TYPE" ];then
    	LIST_CMD="$TAR t -f $BLOB_FILE_NAME"
		EXTRACT_CMD="$TAR x -f $BLOB_FILE_NAME -C $TMP_WORK_DIR"
		EXTRACT_STDOUT_CMD="$TAR x -f $BLOB_FILE_NAME --to-stdout"
	elif [ "$FILE_TYPE" = "$ZIP_FILE_TYPE" ];then
		LIST_CMD="$ZIPINFO -1 $BLOB_FILE_NAME"	
		EXTRACT_CMD="$UNZIP -o -qq $BLOB_FILE_NAME -d $TMP_WORK_DIR"
		EXTRACT_STDOUT_CMD="$UNZIP -qq -p $BLOB_FILE_NAME"
	else
		Response E_DTT_UNKNOWN_BLOB_FORMAT "Non valid file-type:$FILE_TYPE"
	fi

	#extract relevant files into temporary directory
	_ExtractFiles "$LIST_CMD" "$EXTRACT_CMD" "$EXTRACT_STDOUT_CMD"
}

Finalize()
{
	\rm -rf $TMP_WORK_DIR $README_TMP_WORK_FILE $TMP_MD5_FILE
}

_ExtractFiles()
{
	LIST_CMD=$1
	EXTRACT_CMD=$2
	EXTRACT_STDOUT_CMD=$3
	LIST_FILES=`$LIST_CMD 2>/dev/null`

    if [ $? -ne 0 ];then
        Response E_DTT_EXTRACT_CMD_FAILED "Can't perform $LIST_CMD"
    fi
	RELEVANT_FILES=`echo $LIST_FILES | tr " " "\n" | $EGREP "$RELEVANT_FILES_REGEX"`
	README=`echo $LIST_FILES | tr " " "\n" | $EGREP "$README_REGEX"`

	PATCHNAME=`echo $LIST_FILES | tr " " "\n" | $EGREP "$PATCH_REGEX" | head -n 1| cut -f1 -d'/'`
	if [ -z "$PATCHNAME" ]; then
		Response E_DTT_PATCH_DIR_NOT_FOUND "Patch directory not found"
	fi
	PATCHID=`echo $PATCHNAME | cut -f1 -d'-'`
	
	#extract signature files	
	if [ ! -z "$RELEVANT_FILES" ];then
		CMD="$EXTRACT_CMD $RELEVANT_FILES"
		$CMD >/dev/null 2>/dev/null
		if [ $? -ne 0 ];then
			Response E_DTT_EXTRACT_CMD_FAILED "Can't perform $CMD"
    	fi
	fi

	#extract README into temporary file
    if [ ! -z "$README" ];then
        CMD="$EXTRACT_STDOUT_CMD $README"
        $CMD > $README_TMP_WORK_FILE 2>/dev/null
		if [ $? -ne 0 ];then
			Response E_DTT_EXTRACT_CMD_FAILED "Can't perform $CMD"
		fi
    fi
}

ParseReadme()
{
	#create directory if it wasn't created before
	\mkdir -m 0700 -p $TMP_WORK_DIR/$PATCHNAME
		  
	#Tags should be sorted alpha-numeric
	_GetDepLine "REQUIRES" "$REQUIRES_REGEX" $PATCH_REGEX
	
	_GetDepLine "OBSOLETED" "$OBSOLETED_REGEX" $PATCH_REGEX

	_GetDepLine "OBSOLETES" "$OBSOLETES_REGEX" $PATCH_REGEX

	_GetDepLine "CONFLICTS" "$CONFLICTS_REGEX" $PATCH_REGEX

	_GetLine "OS" "$OS_RELEASE_REGEX" "[^\s]+"
}

_GetDepLine()
{
    buffer=""
    words=`$EGREP "$2" $README_TMP_WORK_FILE | cut -f2 -d':' | sed 's/^ \+//g'`
    for word in $words;
    do
        echo $word | $EGREP $3 >/dev/null
        if [ $? -eq 0 ];then
            fixed=`echo $word | sed "s/.*\($PATCH_SED_REGEX\).*/\1/g"`
            dep_patchid=`echo $fixed | cut -f1 -d'-'`
            if [ "^$dep_patchid^" != "^$PATCHID^" ];then
                buffer="$buffer $fixed"
            fi
        fi
    done
    TMP_LIST=`echo $buffer | tr " " "\n" | sort | tr "\n" " "`

    echo "$1 : $TMP_LIST" | sed 's/ $//' >> $TMP_WORK_DIR/$PATCHNAME/.README
}

_GetLine()
{
    buffer=`$EGREP "$2" $README_TMP_WORK_FILE |  cut -f2 -d':'  | tr " " "\n" | xargs`
    echo "$1 : $buffer" >> $TMP_WORK_DIR/$PATCHNAME/.README
}

CalcMD5Sum()
{
	find $TMP_WORK_DIR/$PATCHNAME/ -type f -exec $MD5SUM $MD5SUM_ARGS {} \; | cut -f1 -d ' ' | sort > $TMP_MD5_FILE
	FINAL_MD5SUM=`$MD5SUM $MD5SUM_ARGS $TMP_MD5_FILE | cut -f1 -d' '`
}

################MAIN#######################

Main()
{
	if [ "^${1}^" != "^-blob_file_name^" ];then
		Response E_DTT_BAD_USAGE "$USAGE"
	fi

	if [ -z $2 ];then
		Response E_DTT_BAD_USAGE "$USAGE"
	fi

	BLOB_FILE_NAME=$2

	#Assert all required utilities are available
	FirstAssert

	if [ ! -f "$BLOB_FILE_NAME" ] ; then
		Response E_DTT_MISSING_BLOB_FILE "$BLOB_FILE_NAME, file not found!"
	fi

	if [ ! -s "$BLOB_FILE_NAME" ] ; then
		Response E_DTT_EMPTY_BLOB_FILE "$BLOB_FILE_NAME, empty file!"
	fi

	if [ ! -r "$BLOB_FILE_NAME" ] ; then
		Response E_DTT_NO_PERMISSIONS "$BLOB_FILE_NAME, no permissions!"
	fi

	rm -rf $TMP_WORK_DIR
	\mkdir -m 0700 -p $TMP_WORK_DIR

	#Identify file-type of blob, set FILE_TYPE
	Identify $BLOB_FILE_NAME

	#Main method to handle recieved blob
	HandleBlob $BLOB_FILE_NAME $FILE_TYPE

	if [ -f $README_TMP_WORK_FILE ];then
		#If found readme, parse it for relevant dependencies.
		ParseReadme
	fi

	CalcMD5Sum
	echo "$FINAL_MD5SUM"

#	Response $OS_S_OK "" "$FINAL_MD5SUM"
}

Main $*
