#!/bin/bash 
#
# Copyright (c) 2007, 2011 Oracle and/or its affiliates.  All rights reserved.
# Use is subject to license terms.
#
#
# This script downloads the required data and files from the Sun Connection
# Knowledge Base servers to allow for operation of the Sun Ops Center
# product in a "standalone" (i.e. no internet access) mode.  
# The execution of this script results in the creation a tarball which 
# can be copied over to the standalone server and extracted.

export PATH=$PATH:/bin:/usr/sbin:/usr/bin

scriptDir=$(dirname $0)
cd $scriptDir
if [ "$scriptDir" = "." ]; then
	scriptDir=`pwd`
fi
umask 0022

############### Read config #################
export TEMP_CONFIG_VARS_FILE="/var/tmp/harvester-vars.$$"
if [ -f ./config.ini ]; then
	chmod 700 config.ini
	rm -f $TEMP_CONFIG_VARS_FILE >/dev/null 2>/dev/null		# cleanup just in case
	cat ./config.ini | grep -v "^#" | while read line; do
		if [ ! "$line" = "" ]; then
			PARAM=`echo "$line" | cut -d'=' -f1`
			VAL=`echo "$line" | cut -d'=' -f2- | sed -e 's/\"//g'`		### remove " from it
			if [ "$PARAM" = "HTTP_PASSWD" ]; then		### minor correction to make config.ini more user friendly
				PARAM="HTTP_PASSWD_UNESCAPED"
				if [ -f "$VAL" ]; then			##### Allow specifying a filename instead of password
					echo "export MSG_DISPLAY1=\"[OK] Reading MOS password from $VAL\"" >> $TEMP_CONFIG_VARS_FILE
                                        echo "export CFG_PWD_FILE=\"$VAL\"" >> $TEMP_CONFIG_VARS_FILE
					read -r VAL < ${VAL}
					if [[ -z "${VAL// }" ]]; then
					    echo "export EMPTY_PASSWORD_FOUND=\"Empty password found\"" >> $TEMP_CONFIG_VARS_FILE
					fi
				fi
			fi
			if [ "$PARAM" = "PROXY_PASSWD" ]; then		### minor correction to make config.ini more user friendly
				PARAM="PROXY_PASSWD_UNESCAPED"
				if [ -f "$VAL" ]; then			##### Allow specifying a filename instead of password
					echo "export MSG_DISPLAY2=\"[OK] Reading Proxy password from $VAL\"" >> $TEMP_CONFIG_VARS_FILE
                                        echo "export CFG_PROXY_PWD_FILE=\"$VAL\"" >> $TEMP_CONFIG_VARS_FILE
					read -r VAL < ${VAL}
					if [[ -z "${VAL// }" ]]; then
					    echo "export EMPTY_PROXY_PASSWORD_FOUND=\"Empty proxy password found\"" >> $TEMP_CONFIG_VARS_FILE
					fi
				fi
			fi
			
			ENVPARAM=`eval "echo \\$${PARAM}"`              ### the current value of this parameter
			#echo "PARAM:$PARAM VAL:$VAL ENVPARAM:$ENVPARAM"
			if [ "$ENVPARAM" = "" ]; then           # if not set already
				echo "export $PARAM=$VAL" >> $TEMP_CONFIG_VARS_FILE
			fi
		fi
	done
fi
if [ -f $TEMP_CONFIG_VARS_FILE ]; then
        chmod 700 $TEMP_CONFIG_VARS_FILE
	### Need this since bash creates a subshell for the while loop and 'forgets' the variables
	source $TEMP_CONFIG_VARS_FILE
	rm -f $TEMP_CONFIG_VARS_FILE
fi
############### End of Read config #################

## -------------------------- ##
##       Variables            ##
## -------------------------- ##
DEBUG=false						### Debug for harvester.sh, should be false in PRODUCTION
RUNSTAGE1=true					### Need to be true in production (false will skip KB download and Proxy modify - use for debugging only!)
MAX_THREADS=8					### Higher limit is faster, but may block you out of MOS due to too many connections
MAX_THREADS_OEL=4				### Number of allowed threads for OEL only

## -------------------------- ##
##    Params: Do not change   ##
## -------------------------- ##

START_TIME=`date`
TAR_DATE=`date '+%m%d'`

NO_OF_PATCHES=0
PASSWORD_FILE=""
VERSION_FILE="harvester-latest.version"
MIRROR1="${KB}/OCDoctor"
MIRROR1_NAME="${KB}"
EXCLUDE_FOLDER="Do_not_copy_this_folder_to_destination_ec"

######################### Linux / Solaris config #################
UNAME=`uname`
if [ $UNAME = "Linux" ];then
	OPENSSL=/usr/bin/openssl
	EPOCH=`date +%s`
	GREP=/bin/grep
	EGREP=/bin/egrep
	GTAR=/bin/tar
	CHECK_UTILS="/usr/bin/wget $OPENSSL $GREP $GTAR $EGREP /usr/bin/pgrep /bin/gzip /bin/sed /usr/bin/wc /bin/awk /bin/sort /usr/bin/head /usr/bin/tail /usr/bin/less /usr/bin/tr /bin/egrep /usr/bin/perl"
	WGET="/usr/bin/wget"
	DF="df -Pk"
	MD5SUM="/usr/bin/md5sum"
    MD5SUM_ARGS=""
    TAC="/usr/bin/tac"
else
	OPENSSL=/usr/sfw/bin/openssl
	EPOCH=`nawk 'BEGIN{print srand()}'`
	GREP=/usr/sfw/bin/ggrep
	EGREP=/usr/bin/egrep
	GTAR=/usr/sfw/bin/gtar
	CHECK_UTILS="/usr/sfw/bin/wget $OPENSSL $GREP $GTAR $EGREP /usr/bin/pgrep /usr/bin/gzip /usr/bin/sed /usr/bin/wc /usr/bin/awk /usr/bin/sort /usr/bin/head /usr/bin/tail /usr/bin/less /usr/bin/tr /usr/bin/egrep /usr/bin/perl"
	WGET="/usr/sfw/bin/wget"
	DF="df -k"
	MD5SUM="/bin/digest"
	MD5SUM_ARGS="-a md5"
	TAC="/usr/bin/tail -r"
fi

#
# Check which OS we are and architecture
#

DOWNLOAD_PATCHES=0
DOWNLOAD_PATCHES_FOR=""
DOWNLOADED_PATCHES_FOR=""

#
# Create a new wget log file each run.
#
if [ -f ${WGET_LOG} ]; then
    rm ${WGET_LOG}
    touch ${WGET_LOG}
fi

## -------------------------- ##
##    Setup and Checks        ##
## -------------------------- ##

#
# Check for the presence of the utilities we need
#
for util in ${CHECK_UTILS}; do
    if [ ! -f ${util} ]; then
        echo "Error: cannot locate required utility ${util}"
        exit 1
    fi
done

#
# Functions
#

compare_file_permissions(){
    FILENAME=$1
    PERM=`perl -le' printf "%o\n", 07777 & (stat)[2] for @ARGV ' $FILENAME`
    if [ "$PERM" != "400" ]; then
       echo ""
       echo "WARNING: found possible insecure password file permissions: FILE=$FILENAME PERMISSIONS=$PERM"
       echo "if you choose to continue the script will change the file permissions to 400,"
       echo "or you can quit and take the necessary actions to protect your password before reruning the script."
       echo "Would you like to continue? Enter 'yes' or 'no' and press [ENTER]"
       read choice
       if [ "$choice" == "no" ]; then
           exit 1
       elif [ "$choice" == "yes" ]; then
           chmod 400 $FILENAME
           echo "changed permissions of password file '$FILENAME' to 400. Continuing."
       else
           echo "Error: non valid answer was entered. Exiting"
           exit 1
       fi
    fi
}	

create_signatures_file(){
	sync
	sleep 1
	CHANNEL=$1
	cd ${CONFIG_FILES}/${CHANNEL}
	
	if [ ! -f proxy.tgz ]; then
		echo "ERROR: create_signatures_file: Cannot find proxy.tgz in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
		cd ${STANDALONE_DIR}
		return 1
	fi
	if [ ! -f proxy.tgz.sig ]; then
		echo "ERROR: create_signatures_file: Cannot find proxy.tgz.sig in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
		cd ${STANDALONE_DIR}
		return 1
	fi
	#if [ ! -f proxy.tgz.sig.OK ]; then
	#	echo "ERROR: create_signatures_file: Cannot find proxy.tgz.sig.OK in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
	#	cd ${STANDALONE_DIR}
	#	return 1
	#fi

	# This one is always good to cleanup before downloading anything
	rm -rf temp_${CHANNEL} 
	mkdir temp_${CHANNEL} 
	sync
	sleep 1
	cd temp_${CHANNEL}

	#extract the proxy.tgz file
	sync
	sleep 1
	if [ "$CHANNEL" = "FIRMWARE" ]; then
		${GTAR} zxf ../proxy.tgz osp_catalog.xml
		if [ -f osp_catalog.xml ]; then
			##### Patch name | Blob ID | Digest (md5) | Size ######
			${GREP} DigestHash osp_catalog.xml | awk -F"\"" '{printf("%s|%s\n",$4,$8)}' > ../signatures-firmware.txt
			echo "OK: Created signatures file for channel $CHANNEL"
		else
			echo "ERROR: create_signatures_file: Cannot extract osp_catalog.xml in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
			cd ${STANDALONE_DIR}
			return 1
		fi
	else
		${GTAR} zxf ../proxy.tgz download.xml
		if [ -f download.xml ]; then
			##### Patch name | Signature ######
			#$GREP -A 1 -B 10 BSIG vtree.xml  | $EGREP "<NTS>|<BSIG>" |  tr "\\n" "," | sed  's/<\/BSIG>,/\n/g' | sed -e 's/.*\[\(.*\)\]\].*,.*\[\(.*\)\]\].*/\1.zip|\2/' > ../signatures-solaris.txt
			#${GREP} -A 1 -B 10 "__NAME__" download.xml  | ${EGREP} "__NAME__|FINGER_PRINT" |tr "\\n" "," | sed  's/<\/VAL>,/\n/g' | sed -e 's/.*\[\(.*\)\]\].*,.*<VAL>\(.*\)/\2.zip|\1/'  > ../signatures-solaris.txt
			${GREP} -A 1 -B 10 "__NAME__" download.xml  | ${EGREP} "__NAME__|FINGER_PRINT" | sed -e 's/<\/VAL>/=/g' | tr "\\n" "," | tr "=" "\\n" | sed -e 's/.*\[\(.*\)\]\].*,.*<VAL>\(.*\)/\2.zip|\1/' > ../signatures-solaris.txt
			echo "OK: Created signatures file for channel $CHANNEL"
		else
			echo "ERROR: create_signatures_file: Cannot extract download.xml in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
			cd ${STANDALONE_DIR}
			return 1
		fi		
	fi
	sync
	sleep 1
	cd ${CONFIG_FILES}/${CHANNEL}
	rm -rf temp_${CHANNEL}
	cd ${STANDALONE_DIR}
	sync 
}	

create_oel_file(){
	sync
	sleep 1
	CHANNEL=$1
	cd ${CONFIG_FILES}/${CHANNEL}
	
	if [ ! -f proxy.tgz ]; then
		echo "ERROR: create_oel_file: Cannot find proxy.tgz in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
		cd ${STANDALONE_DIR}
		return 1
	fi
	if [ ! -f proxy.tgz.sig ]; then
		echo "ERROR: create_oel_file: Cannot find proxy.tgz.sig in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
		cd ${STANDALONE_DIR}
		return 1
	fi

	# This one is always good to cleanup before downloading anything
	rm -rf temp_${CHANNEL} 
	mkdir temp_${CHANNEL} 
	sync
	sleep 1
	cd temp_${CHANNEL}

	#extract the proxy.tgz file
	sync
	sleep 1
	${GTAR} zxf ../proxy.tgz download.xml
	if [ -f download.xml ]; then
		##### Patch name | Signature ######
		#$GREP -A 1 -B 10 BSIG vtree.xml  | $EGREP "<NTS>|<BSIG>" |  tr "\\n" "," | sed  's/<\/BSIG>,/\n/g' | sed -e 's/.*\[\(.*\)\]\].*,.*\[\(.*\)\]\].*/\1.zip|\2/' > ../signatures-solaris.txt
		#${GREP} -A 1 -B 10 "__NAME__" download.xml  | ${EGREP} "__NAME__|FINGER_PRINT" |tr "\\n" "," | sed  's/<\/VAL>,/\n/g' | sed -e 's/.*\[\(.*\)\]\].*,.*<VAL>\(.*\)/\2.zip|\1/'  > ../signatures-solaris.txt
		${GREP} -A 1 -B 10 "http" download.xml | ${EGREP} "http|FINGER_PRINT"| awk -F"[" '{print $3}' | awk -F"]" '{print $1}' | ${TAC} > ../signatures-oel.txt
		echo "OK: Created signatures file for channel $CHANNEL"
	else
		echo "ERROR: create_oel_file: Cannot extract download.xml in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
		cd ${STANDALONE_DIR}
		return 1
	fi		

	sync
	sleep 1
	cd ${CONFIG_FILES}/${CHANNEL}
	rm -rf temp_${CHANNEL}
	cd ${STANDALONE_DIR}
	sync 
}

edit_download_methods_xml(){				#### Used to modify "Auth" to "Guus" - known issue with OC
	sync
	sleep 1
	METHOD_DEBUG=false
	CHANNEL=$1
	CURRENTFOLD=`pwd`
	$METHOD_DEBUG && echo "Current folder: $CURRENTFOLD"
	
	$METHOD_DEBUG && echo "cd ${CONFIG_FILES}/${CHANNEL}"
	cd ${CONFIG_FILES}/${CHANNEL}
	
	if [ ! -f proxy.tgz ]; then
		echo "ERROR: edit_download_methods_xml: Cannot find proxy.tgz in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
		cd ${STANDALONE_DIR}
		return 1
	fi
	if [ ! -f proxy.tgz.sig ]; then
		echo "ERROR: edit_download_methods_xml: Cannot find proxy.tgz.sig in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
		cd ${STANDALONE_DIR}
		return 1
	fi
	#if [ ! -f proxy.tgz.sig.OK ]; then
	#	echo "ERROR: edit_download_methods_xml: Cannot find proxy.tgz.sig.OK in ${CONFIG_FILES}/${CHANNEL}, try rerunning."
	#	cd ${STANDALONE_DIR}
	#	return 1
	#fi

	# This one is always good to cleanup before downloading anything
	rm -rf temp_${CHANNEL} 
	mkdir temp_${CHANNEL} 
	sync
	sleep 1
	$METHOD_DEBUG && echo "cd temp_${CHANNEL}"
	cd temp_${CHANNEL}

	#extract the proxy.tgz file
	$METHOD_DEBUG && echo "${GTAR} zxf ../proxy.tgz"
	sync
	sleep 1
	${GTAR} zxf ../proxy.tgz
	sync
	sleep 1
	# replace Auth with Guus 7067569
	# To keep the ownership of download_methods.xml
	$METHOD_DEBUG && echo "cp -p download_methods.xml  download_methods.xml.new"
	cp -p download_methods.xml  download_methods.xml.new
	sync
	sleep 1
	$METHOD_DEBUG && echo "sed 's/Auth/Guus/' < download_methods.xml > download_methods.xml.new"
	sed 's/Auth/Guus/' < download_methods.xml > download_methods.xml.new
	sync
	sleep 1
	$METHOD_DEBUG && echo "mv download_methods.xml.new download_methods.xml"
	mv download_methods.xml.new download_methods.xml
	sync
	sleep 1

	# Repack proxy.tgz and remove the intermediate files.
    #$METHOD_DEBUG && echo "mv ../proxy.tgz ../proxy.tgz.orig"
    
    
	sync
	rm -rf ../proxy.tgz.sig 		### We're not saving this since every time harvester runs, it removes it anyways (to force recheck of sig)
	#rm -rf ../proxy.tgz.sig.OK		### We're not saving this since every time harvester runs, it removes it anyways (to force recheck of sig)
	
	$METHOD_DEBUG && echo "ls -A | ${GTAR} -zcf ../proxy.tgz -T -"
	rm -f ../proxy-tmp.tgz		## cleanup
	sync
	sleep 1
	ls -A | ${GTAR} -zcf ../proxy-tmp.tgz -T -			### this will also compres ".files"
	sync
	
	#### Apply
	mkdir -p ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}		### Folder which is excluded from copying
	mv ../proxy.tgz ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/proxy.tgz
	sync
    sleep 1
	mv ../proxy-tmp.tgz ../proxy.tgz			#### In the past, the TAR create command could have been interrupted, causing a bad proxy.tgz leftover. This will protect it
	
	$METHOD_DEBUG && echo "$OPENSSL dgst -binary -sha1 ../proxy.tgz > ../proxy.tgz.sig"
	$OPENSSL dgst -binary -sha1 ../proxy.tgz > ../proxy.tgz.sig
	$METHOD_DEBUG && echo "cd ${CONFIG_FILES}/${CHANNEL} ; rm -rf temp_${CHANNEL} ; cd ${STANDALONE_DIR}"
	cd ${CONFIG_FILES}/${CHANNEL}
	rm -rf temp_${CHANNEL} 
	cd ${STANDALONE_DIR}
	sync
}        

#
# Check options and usage
#
usage() {
    echo "============================== Download options =============================="
    echo "  --kb-only                     Download knowledgebase (metadata) only, no patches"
    echo "  --download-patches [ list ]   Download patches. This will also download the metadata."
    echo "                                By default it downloads patches for all Solaris distributions."
    echo "                                Optionally provide a list of distribution names to limit"
    echo "                                the distributions for which patches are downloaded."
    echo "                                This optional list consists of a quoted space separated"
    echo "                                list of distribution names. Valid distributions:"
    echo "                                                SOLARIS_10_0_SPARC"
    echo "                                                SOLARIS_10_0_X86"
    echo "                                                SOLARIS_9_0_SPARC"
    echo "                                                SOLARIS_8_0_SPARC"
    #echo "                                                OEL_5_0_AMD64"
    #echo "                                                OEL_5_0_IA32"
    #echo "                                                OEL_6_0_AMD64"
    #echo "                                                OEL_6_0_IA32"
    echo "                                                FIRMWARE"
    echo "                                E.g. \"SOLARIS_10_0_SPARC SOLARIS_10_0_X86\""
    echo "  --download-baseline [ ID ]    Download specific baseline"
    echo "                                Note: When this option is used, the \"--download-patches\" is"
    echo "                                mandatory, but only with ONE distribution (see 3rd example)"
    echo "                                NOTE: To see a list of available IDs, do not specify an ID."
    echo "  --revisions <revisions>       Number of revisions of a patch to download"
    echo "                                Default number is 100 (also available in config.ini)"
    echo "  --from-date <mm-YYYY>         Only download patches that are newer than a specific date."
    echo "  --patches-from-file <file>         Download all the patches from a file. Format of the file is"
    echo "                                     one patch per line (without the '.zip' extension)."
    #echo "  --rpms-from-file <file> <channel>  Download all RPMs from a file. Format of the file is"
    #echo "                                     one RPM per line (including the '.rpm' extention)."
    #echo "                                     '<channel>' must be one of the following:"
    #echo "                                     OEL_5_0_IA32, OEL_5_0_AMD64, OEL_6_0_IA32 or OEL_6_0_AMD64"
	echo ""
	echo "  --instructions                Instructions on how to copy data to the disconnected environment."
    echo "                                                                      "
    echo "  --update                      Update the Harvester utility"
    echo ""
    echo "=============== Config options (also available in config.ini) ================"
    echo "  --user <user>                          The MOS user name"
    echo "  --password-file <passwordfile>         A file containing only the MOS password"
    echo "  --proxy-server <proxy url>             URL to the proxy server"
    echo "  --proxy-user <http proxy url>          Proxy user name"
    echo "  --proxy-password-file <proxypassfile>  A file containing only the proxy password"
    echo "  --storage-dir <dir>                    Directory for storing downloads"
    echo "  --mark-new-patches                     When using this flag, the harvester will create a dated folder"
    echo "                                         which includes hard links to all the patches that were downloaded"
    echo "                                         in the current run. All patches will still go to the storage dir but"
    echo "                                         in addition a dated folder will be created with hardlinks pointing to"
    echo "                                         the real patch files."
    echo "                                         This flag is useful in incramental downloads to identify which are the"
    echo "                                         newly downloaded files."
    echo "                                         Using hardlinks will eliminate the need for a full copy of each file."
    echo ""
    echo ""
    echo "================================== Examples ================================="
    echo "Example 1: To download only knowledge, no patches                     "
    echo "$0 --kb-only"
    echo
    echo "Example 2: Show available Baselines for Solaris 10 SPARC"
    echo "           Note: Use the baseline ID from the output in the next example"
    echo "$0 --download-patches SOLARIS_10_0_SPARC --download-baseline"
    echo 
    echo "Example 3: Download Full Baseline for Aug-2011 for s10sparc (around 3.5G)"
    echo "           Note: The baseline ID is the one used from Example 2          "
    echo "$0 --download-patches SOLARIS_10_0_SPARC --download-baseline 40030030"
	echo
    echo "Example 4: To download s10sparc and s10x86 patches up to 4 revs back for each patch"
    echo "$0 --download-patches \"SOLARIS_10_0_SPARC SOLARIS_10_0_X86\" --revisions 4"
    echo 
    echo "Example 5: To download all patches on all channels, but only the latest revisions of each patch (around 50G)"
    echo "$0 --download-patches --revisions 1"
    echo
    echo "Example 6: To download all patches from July 2011 (around 4G), and to mark incremental downloads"
    echo "$0 --download-patches --from-date 07-2011 --mark-new-patches"
    echo 
    echo "Example 7: To download all firmware updates"
    echo "$0 --download-patches FIRMWARE"
    echo 
    exit 1
}

# ----------------- Self Update ----------------
check_for_update(){
    if [ "$1" = "--force" ];then
		FORCED=1
	fi
	
	echo "Trying to download updates..."
	rm -f /var/tmp/${VERSION_FILE} >/dev/null 2>/dev/null		### cleanup before downloading
	WGET_OUTPUT=`./download.sh "Harvester version file" "${MIRROR1}/${VERSION_FILE}" ./ 2>&1`
	if [ $? -ne 0 ];then
		echo "ERROR: Unable to connect to ${KB}. Please check config.ini and the file ${WGET_LOG}"
		echo "       Output of the download command is:"
		printf "%s\n" "$WGET_OUTPUT"
		echo ""
		exit 1
	else
		echo ""
		echo "OK: Version file was downloaded successfully"
		echo ""
	fi
	#rm -f ${VERSION_FILE}.OK >/dev/null 2>/dev/null
	## Massage VERSION_FILE and compare to current version
	if [ $UNAME = "Linux" ];then
		/usr/bin/dos2unix --quiet $VERSION_FILE 2>/dev/null
	else
		/usr/bin/dos2unix $VERSION_FILE > ${VERSION_FILE}_fixed 2>/dev/null
		mv -f ${VERSION_FILE}_fixed $VERSION_FILE
	fi
	ONLINEVER=`head -1 $VERSION_FILE`
	if [ "$ONLINEVER" = "" ]; then
		echo "ERROR: Downloaded version file OK but unable to determine version number from it."
		echo "---- Begin file content ----"
		cat $VERSION_FILE
		echo "---- End file content   ----"
		echo "It may be a temporary error, try again in a minute."
		echo
		rm -f $VERSION_FILE 2>&1 >/dev/null						### Done with the version file
		exit 1
	fi
	rm -f $VERSION_FILE 2>&1 >/dev/null						### Done with the version file
	### Remove - and . ###
	ONLINEVERNUM=`echo $ONLINEVER | sed -e 's/\.//g' | sed -e 's/-//g'`
	VERSIONNUM=`echo $VERSION | sed -e 's/\.//g' | sed -e 's/-//g'`
	if [ $ONLINEVERNUM -gt $VERSIONNUM ] || [ "$FORCED" = "1" ]; then
		if [ "$FORCED" = "1" ]; then
			echo "OK: No updates were found but '--force' was selected (current: $VERSION, online: $ONLINEVER)"
		else
			echo "OK: Found an update for the Harvester (current: $VERSION, online: $ONLINEVER)."
		fi
	else
		echo "OK: No updates were found (current: $VERSION, online: $ONLINEVER)."
		echo "    You may use \"--update --force\" to force an upgrade."
		echo 
		exit 0
	fi
	
	DOWNLOADED_FILENAME="harvester_bundle-${ONLINEVER}.zip"
	
	MIRROR1_URL=${MIRROR1}/${DOWNLOADED_FILENAME}
	
	rm -rf ../harvester_tmp/
	mkdir ../harvester_tmp/
	sync
	
	echo ""
	echo "Downloading $DOWNLOADED_FILENAME file using mirror: $MIRROR1_NAME"
	./download.sh "Harvester" "$MIRROR1_URL" ../harvester_tmp/ 1>/dev/null 2>/dev/null
	RET_WGET=$?
	if [ $RET_WGET -ne 0 ]; then
		echo "ERROR: Download failed, cannot upgrade."
		echo ""
		exit 1
	fi
	echo ""
	echo "OK: $DOWNLOADED_FILENAME was downloaded successfully"
	
	cd ../harvester_tmp/
	#rm -f $DOWNLOADED_FILENAME.OK >/dev/null 2>/dev/null
	#### We're now in ../harvester_tmp/
	
	###### SANITY TESTS before upgrading and before removing current Harvester folder
	sync
	unzip -qq $DOWNLOADED_FILENAME >/dev/null 2>/dev/null
	if [[ ! $? -eq 0 ]]; then
		echo "ERROR: Unable to extract ZIP file $DOWNLOADED_FILENAME, cannot upgrade."
		echo ""
		exit 1
	fi
	if [ ! -f harvester_bundle/harvester.sh ]; then		#### Safety to make sure we have a proper ZIP file!
		echo "ERROR: Unable to extract ZIP file $DOWNLOADED_FILENAME ('harvester.sh' is not inside of it), cannot upgrade."
		echo ""
		exit 1
	fi
	#### Check if we have preupgrade script
	if [ -f harvester_bundle/pre-upgrade.sh ]; then
		harvester_bundle/pre-upgrade.sh
		if [[ ! $? -eq 0 ]]; then
			echo "ERROR: The pre-upgrade script from version $ONLINEVER failed, not performing the upgrade."
			echo ""
			echo "You may look at this script here:"
			echo "$scriptDir/../harvester_tmp/harvester_bundle/pre-upgrade.sh"
			echo ""
			exit 1
		fi
	fi
	
	cd ..
	# NO NEED TO DELETE CURRENT HARVESTER # rm -rf harvester_bundle/*
	#unzip -qq harvester_tmp/$DOWNLOADED_FILENAME
	sync
	unzip -oqq harvester_tmp/$DOWNLOADED_FILENAME
	rm -rf harvester_tmp
	cd harvester_bundle
	if [ -f pre-upgrade.sh ]; then
		rm -f pre-upgrade.sh
	fi
	if [ -f post-upgrade.sh ]; then
		./post-upgrade.sh
		if [[ ! $? -eq 0 ]]; then
			echo "ERROR: Upgrade finished but the post-upgrade script from version $ONLINEVER failed."
			echo ""
			echo "You may look at this script here:"
			echo "$scriptDir/post-upgrade.sh"
			echo ""
		else
			rm -f ./post-upgrade.sh
		fi
	fi

	VERSION=`head -1 version`
	VDATE=`tail -1 version`
	echo "OK: Updated successfully to version $VERSION ($VDATE)."
	echo
	echo "Please view the CHANGELOG file for list of changes."
	echo
	
	exit 0
}

print_instructions(){
	TDATE=$1
	if [ "$TDATE" = "" ]; then
		TDATE="[timestamp]"
	fi
    echo "INSTRUCTIONS: HOW TO COPY DATA TO THE DISCONNECTED ENVIRONMENT"
    echo "=================================================================="
    echo ""
    echo "METADATA ONLY (--kb-only flag)"
    echo "-----------------------------------"
    echo "Copy ${STANDALONE_DIR}/standalone-${TDATE}.tar.gz to a disconnected Ops Center Enteprise Controller and "
    echo "use the Web UI to configure Update Disconnected Mode:"
    echo "1) Go to Administration"
    echo "2) Click \"Setup connection mode\" action"
    echo "3) Click \"Browse\", and choose the file in the folder where you put the tar.gz file, click \"Load Bundle\""
    echo "4) If the EC is in \"Connected mode\", click \"Switch to Disconnected Mode\""
    echo ""
    echo "To update the OCDoctor on the Enterprise Controller, follow the "METADATA + PATCHES" section below as well."
    echo ""
    echo "METADATA + PATCHES"
    echo "----------------------"
    echo "Copy ${STANDALONE_DIR}/standalone-${TDATE}.tar.gz to a disconnected Ops Center Enteprise Controller."
    echo "In addition, copy ${STANDALONE_DIR}/all_unsigned/ folder which includes all patches. You can split"
    echo "this folder accross multiple DVDs if needed."
    echo ""
    echo "On the Disconnected EC, FIRST FOLLOW THE \"METADATA ONLY\" instuctions above."
    echo ""
    echo "Next, inside the \"all_unsigned\" folder there is a script called \"copy_patches_to_oc.sh\""
    echo "Simply run the script and it will copy the patches to the right location. So on the disconnected EC, run:"
    echo "# <path_to_disconnected_media>/all_unsigned/copy_patches_to_oc.sh"
    echo ""
    echo "IMPORTANT Ops Center 11g users: prior to update 3, each time you use the Web UI to upload data, the \"local-kb\" folder"
    echo "will be cleared. If you are using 11g update 2 and lower (1559), you may want to backup \"all_unsigned\" folder in the"
    echo "\"/var/opt/sun/xvm/uce/opt/server/local-kb/\" folder before uploading anything through the Web UI."
    echo "12c users do not need to perform this."
    echo ""
    echo "INCRAMENTAL PATCHES"
    echo "----------------------"
    echo "This option is only available if \"--mark-new-patches\" was used or if \"MARK_NEW_PATCHES=1\" was set in config.ini ."
    echo "Instructions are identical to \"METADATA + PATCHES\", except instead of copying \"all_unsigned\", you"
    echo "may copy the dated folder which includes patches downloaded today. For example, \"all_unsigned-2011-Sep-14\"."
    echo "Simply run \"copy_patches_to_oc.sh\" from that folder on the disconnected EC to copy those patches."
	echo ""
}

check_max_threads(){
	CURRENT_THREADS=`pgrep -P $$ download.sh 2>/dev/null | wc -l`
	while [ ! $CURRENT_THREADS -lt $MAX_THREADS ]; do			### wait here until we get a clear thread
		$DEBUG && echo "Sleeping... CURRENT_THREADS: $CURRENT_THREADS"
		sleep 1
		CURRENT_THREADS=`pgrep -P $$ download.sh 2>/dev/null | wc -l`
	done
}
check_max_threads_oel(){
	CURRENT_THREADS=`pgrep -P $$ download-oel.sh 2>/dev/null | wc -l`
	while [ ! $CURRENT_THREADS -lt $MAX_THREADS_OEL ]; do			### wait here until we get a clear thread
		$DEBUG && echo "Sleeping... CURRENT_THREADS: $CURRENT_THREADS"
		sleep 1
		CURRENT_THREADS=`pgrep -P $$ download-oel.sh 2>/dev/null | wc -l`
	done
}
wait_for_threads_to_finish(){
	$DEBUG && echo "---------------------------------------- Before checking running threads "
	#### wait for all threads to finish
	CURRENT_THREADS=`pgrep -P $$ download.sh 2>/dev/null | wc -l`			#### look for download.sh from this parent ID only
	COUNT=0
	while [ $CURRENT_THREADS -gt 0 ]; do			### wait here until we get a clear thread
		$DEBUG && echo "Waiting for threads to finish... CURRENT_THREADS:$CURRENT_THREADS"
		sleep 1
		let COUNT=${COUNT}+1
		if [ $COUNT -gt 180 ]; then		### Display this every X seconds
			CDATE=`date +"%H:%M:%S%p %Z"`
			echo "$CDATE - Waiting for the following processes to finish downloading:"
			pgrep -P $$ download.sh
			COUNT=0
		fi
		CURRENT_THREADS=`pgrep -P $$ download.sh 2>/dev/null | wc -l`
	done
	CURRENT_THREADS=`pgrep -P $$ download-oel.sh 2>/dev/null | wc -l`			#### look for download.sh from this parent ID only
	COUNT=0
	while [ $CURRENT_THREADS -gt 0 ]; do			### wait here until we get a clear thread
		$DEBUG && echo "Waiting for threads to finish... CURRENT_THREADS:$CURRENT_THREADS"
		sleep 1
		let COUNT=${COUNT}+1
		if [ $COUNT -gt 180 ]; then		### Display this every X seconds
			CDATE=`date +"%H:%M:%S%p %Z"`
			echo "$CDATE - Waiting for the following processes to finish downloading:"
			pgrep -P $$ download-oel.sh
			COUNT=0
		fi
		CURRENT_THREADS=`pgrep -P $$ download-oel.sh 2>/dev/null | wc -l`
	done
	$DEBUG && echo "---------------------------------------- After checking running threads "
}

VERSION=`head -1 version`
VDATE=`tail -1 version`
RDATE=`date +"%d-%b-%Y %H:%M%p %Z"`
print_version() {
	echo "Ops Center Harvester version ${VERSION} ($VDATE) ($UNAME)"
}

print_version
echo "Download log file is located in ${WGET_LOG}"

#
# Process args
#
# Set the default values
SHOW_BASELINE_FLAG=0
DOWNLOAD_BASELINE_FLAG=0
#VALID_CHANNELS="SOLARIS_10_0_SPARC SOLARIS_10_0_X86 SOLARIS_9_0_SPARC SOLARIS_8_0_SPARC OEL_5_0_AMD64 OEL_5_0_IA32 OEL_6_0_AMD64 OEL_6_0_IA32 FIRMWARE"
VALID_CHANNELS="SOLARIS_10_0_SPARC SOLARIS_10_0_X86 SOLARIS_9_0_SPARC SOLARIS_8_0_SPARC FIRMWARE"

# -------- Reading params -------
totalArgs=0
allArgs=$@
while [ ! "$1" = "" ]; do
	#echo "ARGUMENT IS $1"
	case $1 in
		-u | --user )
			shift
			export HTTP_USER=$1
			shift
			;;
                -p | --password-file )
                        shift
                        export PASSWORD_FILE=$1
                        compare_file_permissions $PASSWORD_FILE
                        shift
                        ;;
		-x | --proxy-server )
			shift
			export PROXY_URL=$1
			shift
			;;
		-U | --proxy-user )
			shift
			export PROXY_USER=$1
			shift
			;;			
                -P | --proxy-password-file )
                        shift
                        export PROXY_PASSWORD_FILE=$1
                        compare_file_permissions PROXY_PASSWORD_FILE
                        shift
                        ;;
		-t | --storage-dir )
			shift
			STANDALONE_DIR=$1
			shift
			;;
		-R | --revisions | --revision )
			shift	
			REVISIONS=$1
			shift
			;;
		--from-date )
			shift
			DOWNLOAD_FROM_DATE=$1
			echo "$DOWNLOAD_FROM_DATE" | grep '^[0-9]\{2\}-[0-9]\{4\}' >/dev/null 2>/dev/null
			if [ $? -ne 0 ]; then
				echo "ERROR: Bad date format."
				echo "       Use mm-YYYY, for example: 07-2011"
				echo ""
				exit 1;
			fi
			DOWNLOAD_FROM_DATE_mm=`echo "$DOWNLOAD_FROM_DATE" | awk -F"-" '{print $1}'`
			DOWNLOAD_FROM_DATE_mm=`expr $DOWNLOAD_FROM_DATE_mm - 1`			### Remove "0" from beginning, otherwise perl think it's octal and anyting >8 is an errror. Also month is 00-11
			DOWNLOAD_FROM_DATE_YYYY=`echo "$DOWNLOAD_FROM_DATE" | awk -F"-" '{print $2}'`
			if [ $DOWNLOAD_FROM_DATE_mm -gt 11 ] || [ $DOWNLOAD_FROM_DATE_mm -lt 0 ] || [ $DOWNLOAD_FROM_DATE_YYYY -lt 1990 ] || [ $DOWNLOAD_FROM_DATE_YYYY -gt 2100 ]; then
				echo "ERROR: Bad date range."
				echo "       Use mm-YYYY, for example: 07-2011"
				echo ""
				exit 1;
			fi
			shift
			;;
		-s | --show-baselines | --show-baseline)
			shift
			SHOW_BASELINE=$1
			SHOW_BASELINE_FLAG=1
			shift
			;;
		-b | --download-baseline )
			shift
			if [ "$1" = "" ] && [ ! "$DOWNLOAD_PATCHES_FOR" = "" ]; then			#### no baseline was given, show list instead
				SHOW_BASELINE=${DOWNLOAD_PATCHES_FOR}
				SHOW_BASELINE_FLAG=1
				DOWNLOAD_PATCHES=0
				DOWNLOAD_PATCHES_FOR=""
			else
				DOWNLOAD_BASELINE=$1
				DOWNLOAD_BASELINE_FLAG=1
			fi
			shift
			;;
		-d | --download-patches )
			DOWNLOAD_PATCHES=1			### $2 will have channels
			shift
			if [ ! "$1" = "" ]; then		### $2 not empty
				echo "$1" | grep "^-" >/dev/null 2>/dev/null
				RETVAL=$?
				while [ $RETVAL -ne 0 ] && [ ! "$1" == "" ]; do		### We have a value, not a flag
					if [ "$DOWNLOAD_PATCHES_FOR" = "" ]; then
						DOWNLOAD_PATCHES_FOR="$1"
					else
						DOWNLOAD_PATCHES_FOR="$DOWNLOAD_PATCHES_FOR $1"
					fi
					shift
					echo "$1" | grep "^-" >/dev/null 2>/dev/null
					RETVAL=$?
				done
			fi
			#echo "DOWNLOAD_PATCHES_FOR:$DOWNLOAD_PATCHES_FOR"
			#exit
			;;
		--kb-only | --kb )
			shift
			#### Not much to do, this will increase totalArgs and therefore will download KB only
			;;
		--update | --upgrade )
			shift
			check_for_update $*
			exit 0
			;;
		--mark-new-patches )
			shift
			MARK_NEW_PATCHES=1
			;;
		--patches-from-file )
			shift
			PATCHES_FROM_FILE=$1
			DOWNLOAD_PATCHES=1
			shift
			if [ ! -f $PATCHES_FROM_FILE ]; then
				echo "ERROR: File $PATCHES_FROM_FILE not found"
				echo 
				exit 1
			fi
			;;
		--rpms-from-file )
			shift
			RPMS_FROM_FILE=$1
			DOWNLOAD_PATCHES=1
			shift
			#RPMS_FROM_FILE_CHANNEL=$1
			#shift
			if [ ! -f $RPMS_FROM_FILE ]; then
				echo "ERROR: File $RPMS_FROM_FILE not found"
				echo 
				exit 1
			fi
			#if [ "$RPMS_FROM_FILE_CHANNEL" = "" ]; then
			#	echo "ERROR: When selecting --rpms-from-file, you must specify to which channel those RPMs belong."
			#	echo "       For example, here are the valid channels:"
			#	echo "       # ./harvester.sh --patches-from-file /var/tmp/rpmlist.txt OEL_5_0_IA32"
			#	echo "       # ./harvester.sh --patches-from-file /var/tmp/rpmlist.txt OEL_5_0_AMD64"
			#	echo "       # ./harvester.sh --patches-from-file /var/tmp/rpmlist.txt OEL_6_0_IA32"
			#	echo "       # ./harvester.sh --patches-from-file /var/tmp/rpmlist.txt OEL_6_0_AMD64"
			#	echo ""
			#	exit 1
			#fi
			;;
		--instructions )
			echo
			print_instructions
			echo 
			exit 0
			;;
		-? | -h | --help )
			usage
			exit 0
			;;
		*)
			echo ""
			echo "ERROR: Unknown options '$1'"
			echo ""
			usage
			exit 0
			;;
	esac
	totalArgs=`expr $totalArgs + 1`
done
if [ $totalArgs = 0 ];then
	usage
	exit 0
fi


######## Check for enough disk space #########
AVAILABLE=`$DF . | tail -1 | awk '{print \$4}'`
REQUIRED=1500000		### Same value as the download.sh
if [ $AVAILABLE -lt $REQUIRED ]; then
        echo "Not enough disk space in the current folder - Need ${REQUIRED}K but only have ${AVAILABLE}K" >> ${WGET_LOG}
		echo "Not enough disk space in the current folder - Need ${REQUIRED}K but only have ${AVAILABLE}K"
        exit 1
fi
######## End of Check for enough disk space #########

#
##################  Check for connectivity / updates. ####################
#
echo "Checking for connectivity..."

if [ "${HTTP_USER}" == "" ]; then
	echo "ERROR Must specify MOS user, or specify the user inside the config.ini"
    usage
    exit 1
fi
if [ "${PASSWORD_FILE}" == "" ] && [ "$HTTP_PASSWD_UNESCAPED" = "" ]; then			#### If a file flag wasn't mentioned, and config.ini is empty
	printf "Enter password for ${HTTP_USER}: "
	stty -echo
	read HTTP_PASSWD_UNESCAPED
	stty echo
	printf "\n"
fi

if [[ ! -z $EMPTY_PASSWORD_FOUND ]]; then
    echo "Password was not found in file $CFG_PWD_FILE. Please fix. exiting..."
    exit 1
fi

if [[ ! -z $EMPTY_PROXY_PASSWORD_FOUND ]]; then
    echo "Password was not found in file $CFG_PROXY_PWD_FILE. Please fix. exiting..."
    exit 1
fi

# check and fix password file permissions when defined in config.ini
if [ ! "${CFG_PWD_FILE}" = "" ]; then
	compare_file_permissions $CFG_PWD_FILE
fi
if [ ! "${CFG_PROXY_PWD_FILE}" = "" ]; then
	compare_file_permissions $CFG_PROXY_PWD_FILE
fi

WGETCOOKIES_LOCK_FILE="/tmp/harvester-download-cookies.lock"
COOKIES_FILE="/tmp/harvester-download-cookies.txt"
rm -f /var/tmp/${VERSION_FILE} >/dev/null 2>/dev/null		### cleanup before downloading
rm -f $WGETCOOKIES_LOCK_FILE >/dev/null 2>/dev/null			### cleanup before downloading
rm -f $COOKIES_FILE	>/dev/null 2>/dev/null					### cleanup before downloading
touch $COOKIES_FILE
chmod 700 $COOKIES_FILE

WGET_OUTPUT=`./download.sh "Harvester version file" "${MIRROR1}/${VERSION_FILE}" ./ 2>&1`
if [ $? -ne 0 ];then
	echo "ERROR: Unable to connect to ${KB}. Please check config.ini and the file ${WGET_LOG}"
	echo "       Output of the download command is:"
	printf "%s\n" "$WGET_OUTPUT"
	echo ""
	exit 1
fi
## Massage VERSION_FILE and compare to current version
if [ $UNAME = "Linux" ];then
	/usr/bin/dos2unix --quiet $VERSION_FILE 2>/dev/null
else
	/usr/bin/dos2unix $VERSION_FILE > ${VERSION_FILE}_fixed 2>/dev/null
	mv -f ${VERSION_FILE}_fixed $VERSION_FILE
fi
ONLINEVER=`head -1 $VERSION_FILE`
rm -f $VERSION_FILE 2>&1 >/dev/null						### Done with the version file
#rm -f $VERSION_FILE.OK 2>&1 >/dev/null						### Done with the version file
### Remove - and . ###
ONLINEVERNUM=`echo $ONLINEVER | sed -e 's/\.//g' | sed -e 's/-//g'`
VERSIONNUM=`echo $VERSION | sed -e 's/\.//g' | sed -e 's/-//g'`
if [ "$VERSIONNUM" = "development" ]; then
	VERSIONNUM=9999999999			### prevent upgrading development
fi
if [ "$ONLINEVERNUM" -gt "$VERSIONNUM" ] ; then
	echo ""
	echo "[OK] Connected successfully and found an update for the Harvester (current: $VERSION, online: $ONLINEVER)."
    echo ""
	echo "--------------------------------------------------------------------------------"
	echo ""
	echo "IMPORTANT: It is recommended that you use \"--update\" to upgrade the Harvester."
	echo ""
	echo "--------------------------------------------------------------------------------"
	echo ""
	let x=10
	printf "Waiting 10 seconds"
	while [ $x -gt 0 ]; do
		sleep 1
		printf "."
		let x=$x-1
	done
	printf "\n"
else
	echo "[OK] Connected successfully. No updates were found (current: $VERSION, online: $ONLINEVER)."
fi

### Stats
GDATE=`date "+%Y-%m-%d" 2>/dev/null`
GTIME=`date "+%H:%M:%S" 2>/dev/null`
### Encode ALL Args
allArgs1=`echo "$allArgs" | sed 's/ /_/g' 2>/dev/null`
allArgs=`echo "$allArgs1" | sed 's/\-/%2D/g' 2>/dev/null`
### OS and HARDWARE ###
H_HARDWARE=`uname -p`
H_OS=`uname -s`
MESSAGE="|h1|${GDATE}|${GTIME}|${VERSION}|${H_OS}|${H_HARDWARE}|${HTTP_USER}|${allArgs}"
echo "$MESSAGE" | $OPENSSL enc -aes128 -a -A -salt -pass pass:AAdYQpQy5ltX -out /var/tmp/harvester-message.$$ >/dev/null 2>/dev/null
if [ -s /var/tmp/harvester-message.$$ ]; then
	MESSAGE=`cat /var/tmp/harvester-message.$$`
	MESSAGE=`echo "$MESSAGE" | sed 's/+/%2B/g' 2>/dev/null`
else
	MESSAGE="Empty_harvester_message_file"
fi
rm -f /var/tmp/harvester-message.$$ >/dev/null 2>/dev/null
rm -f /var/tmp/java-cookie.bin > /dev/null 2>/dev/null
PROXYSTR=""
if [ ! "${PROXY_URL}" = "" ];then
	PROXYSTR=${PROXYSTR}" --proxy -e https_proxy=${PROXY_URL} -e http_proxy=${PROXY_URL} "   
	if [ "${PROXY_USER}" != "" ] && [ "${PROXY_PASSWD}" != "" ]; then 
   		PROXYSTR=${PROXYSTR}" --proxy-user=${PROXY_USER} --proxy-passwd=${PROXY_PASSWD}"
	fi
fi
TOKEN=`$WGET -qq $PROXYSTR $CERTFLAG --timeout=10 --tries 2 --save-cookies /var/tmp/java-cookie.bin --keep-session-cookies  https://java.net/projects/oc-doctor/forums/updates/topics/new -O /dev/stdout 2>/dev/null | grep "^var page" 2>/dev/null | awk -F"\"" '{print $4}' 2>/dev/null`
if [ ! "$TOKEN" = "" ]; then
	$WGET -qq $PROXYSTR $CERTFLAG --timeout=10 --tries 2 --load-cookies /var/tmp/java-cookie.bin --post-data="authenticity_token=${TOKEN}&forum_topic%5Bsubject%5D=H1&forum_topic%5Bmessage_format%5D=bbcode&forum_topic%5Bmessage%5D=${MESSAGE}" "https://java.net/projects/oc-doctor/forums/updates/topics" -O /dev/null >/dev/null 2>/dev/null
fi
rm -f /var/tmp/java-cookie.bin > /dev/null 2>/dev/null


##################  Done with updates ####################

#
# Check STANDALONE_DIR here
#
if [ -d ${STANDALONE_DIR} ]; then
	echo "[OK] Using directory ${STANDALONE_DIR}"
	#### Clean old, unused OK files (stopped using those in 2.13)
	find ${STANDALONE_DIR} -name "*.OK" | while read FILETOREMOVE; do
		if [ -f $FILETOREMOVE ]; then
			rm -f $FILETOREMOVE
		fi
	done
	#### Clean markers for new signatures (these are used for "MARK NEW PATCHES")
	find ${STANDALONE_DIR} -name "*.NEWSIG" | while read FILETOREMOVE; do
		if [ -f $FILETOREMOVE ]; then
			rm -f $FILETOREMOVE
		fi
	done
	
else 
    echo "[OK] Directory ${STANDALONE_DIR} not found. This directory will be created."
fi

if [ "$MSG_DISPLAY1" ]; then		### (using MOS password from file mesage)
	echo "$MSG_DISPLAY1"
fi
if [ "$MSG_DISPLAY2" ]; then		### (using proxy password from file mesage)
	echo "$MSG_DISPLAY2"
fi


#
# Define variables that depend on STANDALONE_DIR
#
CHANNELS3="${STANDALONE_DIR}/channels3"
CONFIG_FILES="${STANDALONE_DIR}/config_files"
BLOBS="${STANDALONE_DIR}/blobs"
SCRIPTS_DIR="${BLOBS}/scripts"
PATCHES_DIR="${STANDALONE_DIR}/all_unsigned"
RPMS_DIR="${STANDALONE_DIR}/RPMs"
CHANNELS_XML="${CHANNELS3}/channels.xml"


#
# Validate input
#
NUM_CHANNELS=0


if [ "${DOWNLOAD_PATCHES_FOR}" != "" ]; then
  FOUND=0
  for SELECTED_CHANNEL in $DOWNLOAD_PATCHES_FOR; do
    FOUND=0
    for ALLOWED_CHANNEL in $VALID_CHANNELS; do
       if [ $SELECTED_CHANNEL == $ALLOWED_CHANNEL ]; then
          NUM_CHANNELS=`expr $NUM_CHANNELS + 1`          
          FOUND=1 
          break
       fi
    done
  done
  if [ $FOUND == 0 ]; then
     echo "Error : Invalid channel. Valid channels are : ${VALID_CHANNELS}"
     exit 1
  fi
fi

if [ $REVISIONS -lt 1 ] || [ $REVISIONS -gt 100 ]; then
   echo "Error : Invalid Revisions to download. Valid Range is between 1 and 100"
   exit 1
fi

if [ ${DOWNLOAD_BASELINE_FLAG} -eq 1 ]; then
    if [ ${DOWNLOAD_PATCHES} -eq 0 ] || [ ${NUM_CHANNELS} -ne 1 ]; then
		echo "Error : --download-baseline (-b) option requires --download-patches (-d) to be specified with exactly one distribution"
		exit 1
    fi
fi



## -------------------------- ##
## Create directory structure ##
## -------------------------- ##

mkdir -p ${CHANNELS3} ${CONFIG_FILES} ${BLOBS} ${SCRIPTS_DIR} ${PATCHES_DIR}

### Check if any other downloads are working...
pgrep download.sh >/dev/null 2>/dev/null
if [ $? -eq 0 ]; then
	echo "ERROR: Another download.sh is still running, please wait for it to finish before running the Harvester."
	echo "The process IDs are:"
	pgrep download.sh
	echo
	exit 1
fi
pgrep download-oel.sh >/dev/null 2>/dev/null
if [ $? -eq 0 ]; then
	echo "ERROR: Another download-oel.sh is still running, please wait for it to finish before running the Harvester."
	echo "The process IDs are:"
	pgrep download-oel.sh
	echo
	exit 1
fi


#### Clean up old URL files
rm -f /var/tmp/harvester_url.* >/dev/null 2>/dev/null

## -------------------------- ##
##  Download Channels files   ##
## -------------------------- ##

#rm -f ${CHANNELS3}/channels.xml ${CHANNELS3}/channels.xml.sig ${CHANNELS3}/channels.xml.sig.OK >/dev/null 2>/dev/null			### force sig test
rm -f ${CHANNELS3}/channels.xml ${CHANNELS3}/channels.xml.sig >/dev/null 2>/dev/null			### force sig test
./download.sh "Channels list (channels.xml)" ${KB}/channels3/channels.xml ${CHANNELS3}/ ${KB}/channels3/channels.xml.sig

## -------------------------- ##
##  Download config_files     ##
## -------------------------- ##
if [ "${CHANNELS_LIST}" == "" ];then
    # Extract the text between EXTENSION tags in the channels.xml file
    if [ -f ${CHANNELS_XML} ]; then
    	CHANNELS_LIST=`${GREP} EXTENSION ${CHANNELS_XML} | sed -e 's/<[^>]*>//g' -e 's/SERVICE_CHANNEL/SERVICE/g' | awk '{print $1}' | ${GREP} -v 'PRODUCT_UPDATE'| ${GREP} -v Redhat 2>/dev/null`
    fi
fi

if [ "${CHANNELS_LIST}" == "" ];then
   echo "[ERROR] Channels information could not be downloaded. Please check your MOS credentials. Check ${WGET_LOG} for more information"
   exit 1
fi

## ------------------------------ ##
##  Show baselines for a distro   ##
## ------------------------------ ##
if [ ${SHOW_BASELINE_FLAG} -eq 1 ] && [ ${DOWNLOAD_BASELINE_FLAG} -eq 0 ]; then
	echo "==> Showing baseline options for ${SHOW_BASELINE} <=="
	mkdir -p ${CONFIG_FILES}/${SHOW_BASELINE}
	
	#### For proxy.tgz, we need to only check signature, so we need to remove any sig or OK files
	#rm -f ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz.sig ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz.sig.OK >/dev/null 2>/dev/null
	rm -f ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz.sig >/dev/null 2>/dev/null
	#### need to move original proxy.tgz to this folder to check if signature match
	rm -rf ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz.moved_for_download		### delete leftovers
	if [ -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${SHOW_BASELINE}/proxy.tgz ]; then			## If we have original proxy.tgz saved
		#echo "We have original file in ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${SHOW_BASELINE}/proxy.tgz"
		if [ -f ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz ]; then 				#### should always be there (because we have original so we should have a modified one here
			mv ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz.moved_for_download ### save modified proxy.tgz
			mv ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${SHOW_BASELINE}/proxy.tgz ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz ### move original (unmodified) to the main location so signatures can be compared
		else
			### need to clean up the EXCLUDE FOLDER (probably config_files was cleaned but not the exclude folder)
			rm -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${SHOW_BASELINE}/proxy.tgz >/dev/null 2>/dev/null 
		fi
	fi			
	echo "Downloading Knowledge for ${SHOW_BASELINE}"
	$scriptDir/download.sh "${SHOW_BASELINE} Knowledge" ${KB}/config_files/${SHOW_BASELINE}/proxy.tgz ${CONFIG_FILES}/${SHOW_BASELINE}/ ${KB}/config_files/${SHOW_BASELINE}/proxy.tgz.sig
 
	##### put modified file back in place (Auth/Guus modification) - only needed it for signature check
	
	if [ -f ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz.moved_for_download ]; then #### This file is the modified file. It will will be removed by download.sh if the signatures dont match, otherwise we can reuse it and no need to rebuild it
		#### lets compare the number of files between the modified and unmodified, to make sure it's not corrupt!
		FILES_IN_UNMOD=`${GTAR} ztvf ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz 2>/dev/null | wc -l | awk '{print $1}'`
		FILES_IN_MOD=`${GTAR} ztvf ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz.moved_for_download 2>/dev/null | wc -l | awk '{print $1}'`
		if [ "$FILES_IN_UNMOD" = "$FILES_IN_MOD" ]; then
			echo "OK: Using cached modified download methods for $SHOW_BASELINE"
			mv ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${SHOW_BASELINE}/proxy.tgz	### move original back to excluded folder
			mv ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz.moved_for_download ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz	### Put modified proxy.tgz back in place
		else
			#### we have a broken copy
			rm -f ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz.moved_for_download
			echo "Modifying download methods for $SHOW_BASELINE as cache file was outdated (had $FILES_IN_MOD files in it, but it should have had $FILES_IN_UNMOD)"	
			edit_download_methods_xml ${SHOW_BASELINE}
		fi
	else		### if we didn't have a pre-modified proxy.tgz (either 1-never created or 2-removed since new proxy.tgz was downloaded)
		echo "Modifying download methods for $SHOW_BASELINE"
		edit_download_methods_xml ${SHOW_BASELINE}
	fi 

    # Verify that the correct file arrived...
    if [ -f ${CONFIG_FILES}/${SHOW_BASELINE}/proxy.tgz ]; then
    	echo "Analyzing..."
		echo "Baseline options for ${SHOW_BASELINE}:" > /var/tmp/harvester-baselines.$$
		echo "   ID        Name" >> /var/tmp/harvester-baselines.$$
		cd ${CONFIG_FILES}/${SHOW_BASELINE}
		sync
		sleep 1
		${GTAR} zxf proxy.tgz vtree.xml
		${GREP} -v "USTR" vtree.xml | ${GREP} -A1 -B1 baseline__Baseline | ${GREP} -v "\[CDATA\[0\]"|${GREP} -v NAME |${GREP} -v -- "--" |${GREP} VER -B1 |${GREP} -v -- "--" |sed -e "s%<NID>\([0-9]*\)</NID>%\1: %" -e "s%<VER><\!\[CDATA\[\(.*\)\]\]></VER>%\1#%" | tr -d "\n" | tr "#" "\n" | sed "s/ //g" | awk '{print $1 " " $2}' | ${GREP} Full | sort >> /var/tmp/harvester-baselines.$$
		cd - 1>/dev/null 2>/dev/null
		rm -fr ${CONFIG_FILES}/${SHOW_BASELINE}/vtree.xml
		more /var/tmp/harvester-baselines.$$
		echo
		rm -f /var/tmp/harvester-baselines.$$
		exit 0
    else
		echo "ERROR: Unable to find proxy.tgz for $SHOW_BASELINE."
		echo "Please make sure you provided a correct Distro Name and that download worked."
		echo "Valid names are: SOLARIS_10_0_SPARC SOLARIS_10_0_X86 SOLARIS_8_0_SPARC SOLARIS_9_0_SPARC"
		echo
		exit 1
    fi
fi

##echo -n "[OK] Found the following distributions in the KB: "
#for i in ${CHANNELS_LIST}
#do 
#    echo -n "$i "
#done

#echo




if [ "$PATCHES_FROM_FILE" = "" ] && [ "$RPMS_FROM_FILE" = "" ] && $RUNSTAGE1 ; then		### if not downloading patches from a file
	echo "----- Stage 1  -  Downloading/refreshing required knowledge base (KB) files -----"
	
	## ----------------------- ##
	##  Download proxy.tgz's   ##
	## ----------------------- ##
	
	for CHANNEL in ${CHANNELS_LIST}
	do
	    # create the channel directory in config_files
	    mkdir -p ${CONFIG_FILES}/${CHANNEL} ${BLOBS}/${CHANNEL}
	
	    # download the proxy.tgz for that channel
	    #echo "Downloading KB (proxy.tgz) for channel: ${CHANNEL}"
	
		SOLARIS_CHECK=`echo ${CHANNEL} | ${GREP} "SOLARIS" 2>/dev/null`
		
		#### For proxy.tgz, we need to only check signature, so we need to remove any sig or OK files
		#rm -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.sig ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.sig.OK >/dev/null 2>/dev/null
		rm -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.sig >/dev/null 2>/dev/null
		if [ "$SOLARIS_CHECK" != "" ]; then
			#### need to move original proxy.tgz to this folder to check if signature match
			rm -rf ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.moved_for_download		### delete leftovers
			if [ -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/proxy.tgz ]; then
				#echo "We have original file in ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/proxy.tgz"
				if [ -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz ]; then 				#### should always be there
					mv ${CONFIG_FILES}/${CHANNEL}/proxy.tgz ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.moved_for_download	 ### save modified proxy.tgz
					mv ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/proxy.tgz ${CONFIG_FILES}/${CHANNEL}/proxy.tgz	### move original (unmodified) to the main location so signatures can be compared
				else
					### need to clean up the EXCLUDE FOLDER (probably config_files was cleaned but not the exclude folder)
					rm -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/proxy.tgz >/dev/null 2>/dev/null 
				fi
			fi			
		fi
		
		check_max_threads ""
		# download.sh URL, target folder, SIG_URL
		$scriptDir/download.sh "${CHANNEL} Knowledge" ${KB}/config_files/${CHANNEL}/proxy.tgz ${CONFIG_FILES}/${CHANNEL}/ ${KB}/config_files/${CHANNEL}/proxy.tgz.sig 2>&1 &
		
	done
	
	## -------------------------- ##
	##  Download detect scripts   ##
	## -------------------------- ##
	SCRIPTS_LIST=`${GREP} SCRIPT_URL ${CHANNELS_XML} | sed 's/<[A-Z\/][^>]*>//g' | ${GREP} -v PRODUCT_UPDATE`
	for SCRIPT in $SCRIPTS_LIST; do
		#echo "Downloading the detect script: ${SCRIPT}"
	    #rm -f ${STANDALONE_DIR}/${SCRIPT}.sig ${STANDALONE_DIR}/${SCRIPT}.sig.OK >/dev/null 2>/dev/null
	    rm -f ${STANDALONE_DIR}/${SCRIPT}.sig >/dev/null 2>/dev/null
	    
	    check_max_threads ""
		
		SCRIPTDIRNAME=`dirname "$SCRIPT"`
		SCRIPTNAME=`basename "$SCRIPT"`
		
		$scriptDir/download.sh "Script ${SCRIPTNAME}" ${KB}${SCRIPT} ${STANDALONE_DIR}/${SCRIPTDIRNAME}/ ${KB}${SCRIPT}.sig 2>&1 &
		#$DEBUG && echo "$scriptDir/download.sh \"Detection script ${SCRIPT}\" ${KB}${SCRIPT} ${STANDALONE_DIR}/${SCRIPTDIR}/ ${KB}${SCRIPT}.sig"
	done
	
	sync
	
	## ------------------------------------------------- ##
	##  Download scripts: notifications, seekers, etc    ##
	## ------------------------------------------------- ##
	#echo "Identifying and Downloading the Notifications and Seeker scripts"
	for CHANNEL in ${CHANNELS_LIST};do
	    # check to see if proxy.tgz exists and is a tgz (gzip) file.  An html file means possible download failure
	    if [ -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz ];then
	    	if [ `file ${CONFIG_FILES}/${CHANNEL}/proxy.tgz | ${GREP} gzip | wc -l` == "1" ];then
	    		$DEBUG && echo "Starting with : cd ${CONFIG_FILES}/${CHANNEL} ================"
	            cd ${CONFIG_FILES}/${CHANNEL}
	            # create scratch directories for extraction of proxy.tgz
		    	# This one is always good to cleanup before downloading anything
		    	FOLD=`pwd`
		    	$DEBUG && echo "PWD:$FOLD"
		    	$DEBUG && echo " rm -fr temp_${CHANNEL} "
	            rm -fr temp_${CHANNEL} 
	            $DEBUG && echo "mkdir temp_${CHANNEL} "
	            mkdir temp_${CHANNEL} 
	            sync
	            $DEBUG && echo "cd temp_${CHANNEL}"
	            cd temp_${CHANNEL}
	            
	            FOLD=`pwd`
		    	$DEBUG && echo "PWD:$FOLD"
		    	
	            #extract the proxy.tgz file
	            #echo "Extracting vtree from ${CHANNEL}"
	            sync
	            $DEBUG && echo " ${GTAR} zxf ../proxy.tgz vtree.xml"
	            ${GTAR} zxf ../proxy.tgz vtree.xml
	
	            #
	            # Get list of downloadable patches and download them if 
	            # available in the KB. Note that we do this only for Solaris channels. 
	            # For non-Solaris channels, we download only notifications.
	            # Note this currently does not support "old" channels that don't have a vtree.xml (only vtree)
	            #
	            if [ -f vtree.xml ]; then 
	                
	
		            #
		            # search for notification blobs only 
		            #
		            BLOBS_LIST=`${GREP} -B 1 notify__ vtree.xml | ${GREP} NID | sed 's/<[A-Z\/][^>]*>//g' | awk '{print $1}'`
		            for NOTIFICATION_ID in ${BLOBS_LIST}; do
						check_max_threads ""
						cd $scriptDir
						./download.sh "Blob ${CHANNEL}/${NOTIFICATION_ID}" ${KB}/blobs/${CHANNEL}/${NOTIFICATION_ID}.blob ${BLOBS}/${CHANNEL}/ ${KB}/blobs/${CHANNEL}/${NOTIFICATION_ID}.blob.sig 2>&1 &
						cd ${CONFIG_FILES}/${CHANNEL}/temp_${CHANNEL}
					done
					
		            #
		            # search for all instances of "*.sh" - 
		            # i.e. pkg_seeker.sh, hardware_seeker.sh, etc
		            #
		            # Can we ignore seekers for "metadata only" mode?
		            #
		            FIX_LIST=`${GREP} -B 1 "\.sh" vtree.xml | ${GREP} NID | sed 's/<[A-Z\/][^>]*>//g' | awk '{print $1}'`
		            for FIX_ID in ${FIX_LIST}; do
						check_max_threads ""
						cd $scriptDir
						./download.sh "Fix ${CHANNEL}/${FIX_ID}" ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob ${BLOBS}/${CHANNEL}/ ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob.sig 2>&1 &
						#$DEBUG && echo "$scriptDir/download.sh \"Fix ${CHANNEL}/${FIX_ID}\" ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob ${BLOBS}/${CHANNEL}/ ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob.sig"
						cd ${CONFIG_FILES}/${CHANNEL}/temp_${CHANNEL}
					done
					
		
		            #
		            # search for all instances of "*.so" - i.e. agent modules, etc 
		            #
		            # Can this be ignored in "metadata only" mode?
		            #
		            FIX_LIST=`${GREP} -B 1 "\.so" vtree.xml | ${GREP} NID | sed 's/<[A-Z\/][^>]*>//g' | awk '{print $1}'`
		            for FIX_ID in ${FIX_LIST};do
						check_max_threads ""
						cd $scriptDir
						./download.sh "Fix ${CHANNEL}/${FIX_ID}" ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob ${BLOBS}/${CHANNEL}/ ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob.sig 2>&1 &
						#$DEBUG && echo "$scriptDir/download.sh \"Fix ${CHANNEL}/${FIX_ID}\" ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob ${BLOBS}/${CHANNEL}/ ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob.sig"
						cd ${CONFIG_FILES}/${CHANNEL}/temp_${CHANNEL}
		            done
		            
		
		            #
		            # search for all instances of "onstage" - 
		            # i.e. onstage-postfix rpm, etc
		            #
		            # 
		            FIX_LIST=`${GREP} -B 1 "onstage" vtree.xml | ${GREP} NID | sed 's/<[A-Z\/][^>]*>//g' | awk '{print $1}'`
		            for FIX_ID in ${FIX_LIST}; do
		                check_max_threads ""
					
						cd $scriptDir
						./download.sh "Fix ${CHANNEL}/${FIX_ID}" ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob ${BLOBS}/${CHANNEL}/ ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob.sig 2>&1 &
						#$DEBUG && echo "$scriptDir/download.sh \"Fix ${CHANNEL}/${FIX_ID}\" ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob ${BLOBS}/${CHANNEL}/ ${KB}/blobs/${CHANNEL}/${FIX_ID}.blob.sig"
						cd ${CONFIG_FILES}/${CHANNEL}/temp_${CHANNEL}
					done
					
				fi			### end of "if [ -f vtree.xml ]"
			fi			### end of "if [ `file ${CONFIG_FILES}/${CHANNEL}/proxy.tgz | ${GREP} gzip | wc -l` == "1" ];then"
	
		    $DEBUG && echo "---------------------------------------- finishing up with ${CONFIG_FILES}/${CHANNEL} "
		    cd ${CONFIG_FILES}/${CHANNEL}
		    rm -rf temp_${CHANNEL}
		fi			### end of "if [ -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz ];then"
	done
	
	#### Download the latest OCDoctor
	cd $scriptDir
	rm -f ${STANDALONE_DIR}/all_unsigned/OCDoctor-latest.zip*
	./download.sh "Latest OCDoctor" ${KB}/OCDoctor/OCDoctor-latest.zip ${STANDALONE_DIR}/all_unsigned/ 2>&1 &
	
	wait_for_threads_to_finish ""	
	
	sync
	sleep 1
	
	echo
	echo "----- Stage 1 completed  -  finished downloading required KB files -----"
	echo 
fi		### Run the above only if PATCHES_FROM_FILE is not specified

#### lets refresh current patches list (to know which ones are newer later)
mkdir -p ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/
if [ -d ${PATCHES_DIR} ]; then
	cd ${PATCHES_DIR}
	ls *.zip 2>/dev/null| ${GREP} -v OCDoctor-latest > ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/patchlist.txt 2>/dev/null
else
	touch ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/patchlist.txt
fi
mkdir -p ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/
if [ -d ${RPMS_DIR} ]; then
	cd ${RPMS_DIR}
	find . -name "*.rpm" 2>/dev/null| ${GREP} -v OCDoctor-latest > ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/rpmlist.txt 2>/dev/null
else
	rm -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/rpmlist.txt 2>/dev/null
	touch ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/rpmlist.txt
fi


### Clean old downloads
rm -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/patchlist-downloaded-due-to-signature.txt	>/dev/null 2>/dev/null

			
#### MODIFY PROXY.TGZ OF SOLARIS  HERE ####
if [ "$PATCHES_FROM_FILE" = "" ] && [ "$RPMS_FROM_FILE" = "" ] && $RUNSTAGE1 ; then		### if not downloading patches from a file
	for CHANNEL in ${CHANNELS_LIST}; do
		
		ISFIRMWARE=`echo "${CHANNEL}" | ${EGREP} "FIRMWARE"`
		if [ ! "$ISFIRMWARE" = "" ]; then
			############# Create signatures
			create_signatures_file ${CHANNEL}
		fi
		
		ISOEL=`echo "${CHANNEL}" | ${EGREP} "^OEL_"`
		if [ ! "$ISOEL" = "" ]; then
			############# Create download links file
			create_oel_file ${CHANNEL}
		fi
		
		############# Modify proxy.tgz to change Auth to Guus ##############
		ISSOLARIS=`echo "${CHANNEL}" | ${EGREP} "SOLARIS"`		### there is no "Auth" method in firmware, so no need to change to Guus
		if [ ! "$ISSOLARIS" = "" ]; then
			############# Create signatures
			create_signatures_file ${CHANNEL}
			if [ -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.moved_for_download ]; then		#### This file is the modified file. It will will be removed by download.sh if the signatures dont match, otherwise we can reuse it and no need to rebuild it
				#### lets compare the number of files between the modified and unmodified, to make sure it's not corrupt!
				FILES_IN_UNMOD=`${GTAR} ztvf ${CONFIG_FILES}/${CHANNEL}/proxy.tgz 2>/dev/null | wc -l | awk '{print $1}'`
				FILES_IN_MOD=`${GTAR} ztvf ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.moved_for_download 2>/dev/null | wc -l | awk '{print $1}'`
				if [ "$FILES_IN_UNMOD" = "$FILES_IN_MOD" ]; then
					echo "OK: Using cached modified download methods for $CHANNEL"
					mv ${CONFIG_FILES}/${CHANNEL}/proxy.tgz ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/proxy.tgz	### move original back to excluded folder
					mv ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.moved_for_download ${CONFIG_FILES}/${CHANNEL}/proxy.tgz	### Put modified proxy.tgz back in place
				else
					#### we have a broken copy
					rm -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.moved_for_download
					echo "Modifying download methods for $CHANNEL as cache file was outdated (had $FILES_IN_MOD files in it, but it should have had $FILES_IN_UNMOD)"	
					edit_download_methods_xml ${CHANNEL}
				fi
			else		### if we didn't have a pre-modified proxy.tgz (either 1-never created or 2-removed since new proxy.tgz was downloaded)
				echo "Modifying download methods for $CHANNEL"	
				edit_download_methods_xml ${CHANNEL}
			fi
		
			#echo "Creating signature for ${CONFIG_FILES}/${CHANNEL}/proxy.tgz"
			$OPENSSL dgst -binary -sha1 ${CONFIG_FILES}/${CHANNEL}/proxy.tgz > ${CONFIG_FILES}/${CHANNEL}/proxy.tgz.sig
		fi
		############################## Done with modifying proxy.tgz ####################
	done
fi

if  [ ${DOWNLOAD_PATCHES} = 1 ]; then
	echo
	echo "----- Stage 2  -  Downloading patches -----"
	echo 
fi

###################### Firmware patches download here ###############
if [ "$PATCHES_FROM_FILE" = "" ] && [ "$RPMS_FROM_FILE" = "" ]; then		### if not downloading patches from a file
	OUTPUT=`echo "${DOWNLOAD_PATCHES_FOR}" | ${GREP} "FIRMWARE" 2>/dev/null`
	if [ "$OUTPUT" != "" ]; then
    	CHANNEL=FIRMWARE
		#### Need to extract vtree.xml to download patches...
		if [ ! -d ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL} ]; then
			mkdir -p ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}
		fi
		sync
		sleep 1
		cd ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}
		#echo "Extracting osp_catalog.xml for ${CHANNEL}"
		if [ ! -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz ]; then
			echo "ERROR: could not find ${CONFIG_FILES}/${CHANNEL}/proxy.tgz"
			echo ""
			exit 1
		fi
		${GTAR} zxf ${CONFIG_FILES}/${CHANNEL}/proxy.tgz osp_catalog.xml download.xml
		sync
		sleep 1
		FIRMWARE_COUNT=`${GREP} "ID=" osp_catalog.xml | wc -l | awk '{print $1}'`
		echo "Downloading $FIRMWARE_COUNT FIRMWARE patches (no filtering is available or applied)"
		
    	FIRMWARE_IDS=`${GREP} "ID=" osp_catalog.xml | sed 's/.*ID="//' | sed 's/".*//'`
    	for FIRMWARE_ID in ${FIRMWARE_IDS}; do
            # Find retrieval method and name value from download.xml
            cd ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}
            #RETRIEVAL_METHOD=`${GREP} -A 15 "<NID>${FIRMWARE_ID}" download.xml | ${GREP} -A 6 '<RETRIEVAL_MTD>' | ${GREP} "<ID>" | sed 's/[><]/ /g' | awk '{print $2;}'`
            NAME_VALUE=`${GREP} -A 15 "<NID>${FIRMWARE_ID}" download.xml | ${GREP} -A 6 '<RETRIEVAL_MTD>' | ${GREP} "<VAL>" | sed 's/[><]/ /g' | awk '{print $2;}'`

            #echo "Downloading ${NAME_VALUE} id ${FIRMWARE_ID} method ${RETRIEVAL_METHOD}"

            # Could try to parse this RETRIEVAL_METHOD out of 
            # download_methods.xml but that is ugly and there only appears to 
            # be three methods
            # 0 https://__SERVER_NAME__:__SERVER_PORT__/osimages/__NAME__
            # 1 https://__SERVER_NAME__:__SERVER_PORT__/fwimages/__NAME__
            # 2 https://__SERVER_NAME__:__SERVER_PORT__/all_unsigned/__NAME__

            #PATCH_SUBDIR="osimages"
            #if [ $RETRIEVAL_METHOD -eq 1 ]; then
            #    PATCH_SUBDIR="fwimages"
            #elif [ $RETRIEVAL_METHOD -eq 2 ]; then
            #    PATCH_SUBDIR="all_unsigned"
           	#else
           	#	echo RETRIEVAL_METHOD is $RETRIEVAL_METHOD
            #fi
            PATCH_SUBDIR="all_unsigned"			##### all firmware patches are all_unsigned
            
            #URL="${KB}/${PATCH_SUBDIR}/${NAME_VALUE}"
            #echo "Downloading ${NAME_VALUE} URL ${URL}..."
            #wget_wrapper -O ${NAME_VALUE} ${URL}
			
			PATCHES_DIR_FIRMWARE="${STANDALONE_DIR}/$PATCH_SUBDIR"
			mkdir -p $PATCHES_DIR_FIRMWARE >/dev/null 2>/dev/null
			sync
            
			# 
            # Begin loop to download patches
            # 
			check_max_threads ""
			
			cd $scriptDir
			
			./download.sh "Firmware ${NAME_VALUE}" ${KB}/${PATCH_SUBDIR}/${NAME_VALUE} ${PATCHES_DIR_FIRMWARE}/ "" ${CONFIG_FILES}/${CHANNEL}/signatures-firmware.txt 2>&1 &
			#echo "./download.sh \"Firmware ${NAME_VALUE}\" ${KB}/${PATCH_SUBDIR}/${NAME_VALUE} ${PATCHES_DIR_FIRMWARE}/ \"\" ${CONFIG_FILES}/${CHANNEL}/signatures-firmware.txt"
			#check_signature ${CONFIG_FILES}/${CHANNEL}/signatures.txt ${PATCHES_DIR_FIRMWARE}/${NAME_VALUE} "FIRMWARE"
			
			let FIRMWARE_COUNT=${FIRMWARE_COUNT}-1
			let TEMP_VALUE=${FIRMWARE_COUNT}%50
			if [ ${TEMP_VALUE} == 0 ] && [ ! "$FIRMWARE_COUNT" = "0" ]; then
			    echo "About ${FIRMWARE_COUNT} remaining to download..."
			fi
        done
        DOWNLOADED_PATCHES_FOR="${DOWNLOADED_PATCHES_FOR} FIRMWARE"
        
        sync
		sleep 1		### allow downloads to finish
				
	fi
fi
###################### End of firmware patches ######################

DOWNLOADED_FROM_FILE=0				##### Only download from a file ONCE... not for each channel (if --patches-from-file was mentioned)
for CHANNEL in ${CHANNELS_LIST}; do
	ISOEL=`echo "${CHANNEL}" | ${GREP} "^OEL_"`
	if [ ! "$ISOEL" = "" ] && [ "$PATCHES_FROM_FILE" = "" ]; then			##### only if OEL AND did not specify patch list
		#############################################################################
		############################# DOWNLOAD OEL PATCHES ##########################
		#############################################################################
        CHANNEL_CHECK="yes"			### mark it to download patches
        if  [ ${DOWNLOAD_PATCHES} = 1 ] && [ "${DOWNLOAD_PATCHES_FOR}" != "" ]; then
            # See if we need to exclude this distribution
            CHANNEL_CHECK=`echo "${DOWNLOAD_PATCHES_FOR}" | ${GREP} "${CHANNEL}" 2>/dev/null`
        fi
        if [ ! "$RPMS_FROM_FILE" = "" ]; then				### if we download from a file
        	if [ "$DOWNLOADED_FROM_FILE" = "0" ]; then			### And it wasn't downloaded yet
        		CHANNEL_CHECK="yes"				### mark it to download patches
        		#DOWNLOADED_FROM_FILE=1			### Prevent it from downloading on the next channel
        	else
        		CHANNEL_CHECK=""					#### Don't download again
        	fi	
        fi
        if  [ "$CHANNEL_CHECK" != "" ] && [ ${DOWNLOAD_PATCHES} = 1 ]; then
			PATCHES_ALL=/var/tmp/patches_all.$$.log
		    PATCHES_SORTED=/var/tmp/patches_sorted.$$.log
		    PATCHES_DOWNLOAD=/var/tmp/patches_download.$$.log
		    TYPE2_IN_VTREE=/var/tmp/type2vtree.list.$$
			if [ "$RPMS_FROM_FILE" = "" ]; then
	        	#### Need to extract RPM list from signarue file
	        	
				sync
				sleep 1
	        	echo "Downloading patches for ${CHANNEL}"
				if [ ! "$DOWNLOAD_FROM_DATE" = "" ]; then
					#### Need to extract vtree.xml to download patches...
					if [ ! -d ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL} ]; then
						mkdir -p ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}
					fi
					sync
					sleep 1
					cd ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}
					echo "Extracting vtree.xml for ${CHANNEL}"
					if [ ! -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz ]; then
						echo "ERROR: could not find ${CONFIG_FILES}/${CHANNEL}/proxy.tgz"
						echo ""
						exit 1
					fi
					rm -f vtree.xml
					${GTAR} zxf ${CONFIG_FILES}/${CHANNEL}/proxy.tgz vtree.xml
					sync
					sleep 1
		    		echo "Downloading patches from $DOWNLOAD_FROM_DATE only"
		    		echo "Generating list of patches and timestamps..."
		    		#${GREP} "<USTR>" vtree.xml | ${GREP} "+1" | sed 's/rpm__//g' |awk -F'[\\[+:\\]]' '{printf("%s %s.%s.rpm\n",$4,$3,$5)}'  |sort -n | grep "^1" > /var/tmp/harvester-${CHANNEL}-timestamps.$$
		    		# use perl instead of awk/sed to make it work on Solaris
		    		${GREP} "<USTR>" vtree.xml | ${GREP} "+1" | sed 's/rpm__//g' | perl -p -e 's/\[|\]|\:|\+/|/g' | awk -F'|' '{printf("%s %s.%s.rpm\n",$4,$3,$5)}'  |sort -n | grep "^1" > /var/tmp/harvester-${CHANNEL}-timestamps.$$
		    		
		    		STARTING_DATE=`perl -e "use Time::Local; print timelocal(0,1,1,1,${DOWNLOAD_FROM_DATE_mm},${DOWNLOAD_FROM_DATE_YYYY});"`
		    		cat /var/tmp/harvester-${CHANNEL}-timestamps.$$ | while read PATCH_LINE ; do
		    			PATCH_TIMESTAMP=`echo "$PATCH_LINE" | cut -d' ' -f1`
		    			if [ $PATCH_TIMESTAMP -ge $STARTING_DATE ]; then
		    				#echo "LINE:$PATCH_LINE   Patch_stamp:$PATCH_TIMESTAMP Start:$STARTING_DATE"
		    				PATCH_NAME=`echo "$PATCH_LINE" | cut -d' ' -f2`
		    				#grep $PATCH_NAME ${STANDALONE_DIR}/config_files/${CHANNEL}/signatures-oel.txt | awk -F"/" '{print $NF}' >> ${PATCHES_DOWNLOAD} 2>/dev/null
		    				echo "$PATCH_NAME" >> ${PATCHES_DOWNLOAD}
		    			#else
		    				#echo "TOO OLD: LINE:$PATCH_LINE   Patch:$PATCH_TIMESTAMP Start:$STARTING_DATE"
		    			fi
		    		done
		    		rm -f /var/tmp/harvester-${CHANNEL}-timestamps.$$
		    		rm -rf vtree.xml
		    		cd $scriptDir
		    	else
		    		#### All rpms
		    		grep "^http" ${STANDALONE_DIR}/config_files/${CHANNEL}/signatures-oel.txt | awk -F"/" '{print $NF}' | sort -u > ${PATCHES_DOWNLOAD}
					# Search for all the patches in vtree for Solaris channels
					#cat ${TYPE2_IN_VTREE} | awk -F"/" '{print $2}' > ${PATCHES_ALL}
	 			fi
	 			# Sort the list without revision number
	 		else		#### download patches from a file
	 			echo "Downloading RPMs from file ${RPMS_FROM_FILE}"	
	 			if [ -f ${RPMS_FROM_FILE} ]; then
					cp -p ${RPMS_FROM_FILE} ${PATCHES_DOWNLOAD}
				elif [ -f $scriptDir/${RPMS_FROM_FILE} ]; then
					cp -p $scriptDir/${RPMS_FROM_FILE} ${PATCHES_DOWNLOAD}
				else
					echo "ERROR: Could not find ${RPMS_FROM_FILE}, try using full path"
					echo ""
					exit
				fi
			fi
            
            # 
            # Begin loop to download patches
            # 
            PATCH_SUBDIR="RPMs"
            sync
            sleep 1
		    NO_OF_PATCHES=`wc -l ${PATCHES_DOWNLOAD} | awk '{print $1}'`
			echo "${NO_OF_PATCHES} RPMs will be downloaded"
			mkdir -p ${RPMS_DIR}/${CHANNEL}
			for PATCH_ID in `cat ${PATCHES_DOWNLOAD}`
		    do
				check_max_threads_oel ""
				cd $scriptDir
				if [ "$RPMS_FROM_FILE" = "" ]; then		#### Not downloading from file, show channel
					#### Try to get the public-yum link if available
					URL=`grep $PATCH_ID ${STANDALONE_DIR}/config_files/${CHANNEL}/signatures-oel.txt | grep "public-yum.oracle.com" | head -1`
					if [ "$URL" = "" ]; then
						URL=`grep $PATCH_ID ${STANDALONE_DIR}/config_files/${CHANNEL}/signatures-oel.txt | head -1`
					fi
					#echo "Downloading $PATCH_ID from $URL"
					if [ "$URL" = "" ]; then
						echo "Unable to download RPM $PATCH_ID - download link is not available in the Ops Center Knowledgebase"
					else
						./download-oel.sh "RPM ${PATCH_ID} for ${CHANNEL}" $URL ${RPMS_DIR}/${CHANNEL} ${CONFIG_FILES}/${CHANNEL}/signatures-oel.txt 2>&1 &
					fi
				else 
					#### Need to determine the URL for this RPM
					URL=`grep $PATCH_ID ${STANDALONE_DIR}/config_files/${CHANNEL}/signatures-oel.txt | grep "public-yum.oracle.com" | head -1`
					#echo "grep $PATCH_ID ${STANDALONE_DIR}/config_files/${CHANNEL}/signatures-oel.txt | grep \"public-yum.oracle.com\" | head -1"
					#echo "$URL"
					
					if [ "$URL" = "" ]; then
						URL=`grep $PATCH_ID ${STANDALONE_DIR}/config_files/${CHANNEL}/signatures-oel.txt | head -1`
						#echo "grep $PATCH_ID ${STANDALONE_DIR}/config_files/${CHANNEL}/signatures-oel.txt | head -1"
						#echo "$URL"
					fi
					if [ "$URL" = "" ]; then
						echo "[${CHANNEL}] RPM $PATCH_ID does not exist in this channel, skipping"
						#echo "ERROR: Unable to download RPM $PATCH_ID - download link is not available in KB of ${CHANNEL}"
						#echo "       Consider first running './harvester.sh --kb' to update KB files and try again"
					else
						./download-oel.sh "RPM ${PATCH_ID} for ${CHANNEL}" $URL ${RPMS_DIR}/${CHANNEL} ${CONFIG_FILES}/${CHANNEL}/signatures-oel.txt 2>&1 &
					fi
				fi
				let NO_OF_PATCHES=${NO_OF_PATCHES}-1
				let TEMP_VALUE=${NO_OF_PATCHES}%50
				if [ ${TEMP_VALUE} == 0 ] && [ ! "$NO_OF_PATCHES" = "0" ]; then
				    echo "About ${NO_OF_PATCHES} remaining to download..."
				fi 
			done
		    rm -f ${PATCHES_ALL}
		    rm -f ${PATCHES_SORTED}
		    rm -f ${PATCHES_DOWNLOAD}
		    rm -f ${TYPE2_IN_VTREE}
            DOWNLOADED_PATCHES_FOR="${DOWNLOADED_PATCHES_FOR} ${CHANNEL}"
		fi		### end of "if  [ ${SOLARIS_CHECK} = 0 ] && [ ${CHANNEL_CHECK} = 0 ] && [ ${DOWNLOAD_PATCHES} = 1 ]; then"
		rm -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/vtree.xml >/dev/null 2>/dev/null
		rm -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/incidents_info.xml >/dev/null 2>/dev/null
		
		####################################################################################
		############################# END OF DOWNLOAD OEL PATCHES ##########################
		####################################################################################
	fi
	
	ISSOLARIS=`echo "${CHANNEL}" | ${GREP} "SOLARIS"`
	if [ ! "$ISSOLARIS" = "" ] && [ "$RPMS_FROM_FILE" = "" ]; then		### only if Solaris AND did not specify RPM list
		#################################################################################
		############################# DOWNLOAD SOLARIS PATCHES ##########################
		#################################################################################
        CHANNEL_CHECK="yes"			### mark it to download patches
        if  [ ${DOWNLOAD_PATCHES} = 1 ] && [ "${DOWNLOAD_PATCHES_FOR}" != "" ]; then
            # See if we need to exclude this distribution
            CHANNEL_CHECK=`echo "${DOWNLOAD_PATCHES_FOR}" | ${GREP} "${CHANNEL}" 2>/dev/null`
        fi
        if [ ! "$PATCHES_FROM_FILE" = "" ]; then				### if we download from a file
        	if [ "$DOWNLOADED_FROM_FILE" = "0" ]; then			### And it wasn't downloaded yet
        		CHANNEL_CHECK="yes"				### mark it to download patches
        		DOWNLOADED_FROM_FILE=1			### Prevent it from downloading on the next channel
        	else
        		CHANNEL_CHECK=""					#### Don't download again
        	fi	
        fi
        if  [ "$CHANNEL_CHECK" != "" ] && [ ${DOWNLOAD_PATCHES} = 1 ]; then
			PATCHES_ALL=/var/tmp/patches_all.$$.log
		    PATCHES_SORTED=/var/tmp/patches_sorted.$$.log
		    PATCHES_DOWNLOAD=/var/tmp/patches_download.$$.log
		    TYPE2_IN_VTREE=/var/tmp/type2vtree.list.$$
			if [ "$PATCHES_FROM_FILE" = "" ]; then
	        	#### Need to extract vtree.xml to download patches...
				if [ ! -d ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL} ]; then
					mkdir -p ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}
				fi
				sync
				sleep 1
				cd ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}
				echo "Extracting vtree.xml for ${CHANNEL}"
				#echo "${GTAR} zxf ${CONFIG_FILES}/${CHANNEL}/proxy.tgz vtree.xml "
				if [ ! -f ${CONFIG_FILES}/${CHANNEL}/proxy.tgz ]; then
					echo "ERROR: could not find ${CONFIG_FILES}/${CHANNEL}/proxy.tgz"
					echo ""
					exit 1
				fi
				${GTAR} zxf ${CONFIG_FILES}/${CHANNEL}/proxy.tgz vtree.xml incidents_info.xml
				sync
				sleep 1
	        	echo "Downloading patches for ${CHANNEL}"
				echo "Generating cache from vtree.xml for ${CHANNEL}"
				#${GREP} -A5 "<NID>" vtree.xml | egrep "<NID>|<NTS>|<TYPE>"| awk '/<NID>/ {print d; d="";} { d=d" "$0} END { print d}' | grep "<TYPE>2</TYPE>" | sed 's/[^0-9,-/]*//g' | awk -F"/" '{printf("%s %s\n",$1,$2)}' > ${TYPE2_IN_VTREE}
				${GREP} -A5 "<NID>" ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/vtree.xml | egrep "<NID>|<NTS>|<TYPE>|<NAME>"| awk '/<NID>/ {print d; d="";} { d=d" "$0} END { print d}' | grep "patch__" | grep "TYPE>2</TYPE>" | grep -v WITHDRAWN | awk '{printf("%s %s\n",$1,$3)}' | sed 's/[^-0-9/]*//g' > ${TYPE2_IN_VTREE}
				$DEBUG && echo "TYPE2_IN_VTREE: ${TYPE2_IN_VTREE}"
						
	            # Code here to check if only a baseline should be downloaded or not
	            touch ${PATCHES_ALL}				### need this in case chose from a date but list is empty
	            touch ${PATCHES_DOWNLOAD}			### need this in case chose from a date but list is empty
	            if [ ${DOWNLOAD_BASELINE_FLAG} = 1 ];then
	                echo "Downloading a baseline only"
	                echo "Generating a patchlist..."
	                # Translate Baseline Blob ID to PatchList
	                for PATCH_ID in `cat ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/vtree.xml | sed -n "/<NID>${DOWNLOAD_BASELINE}<\/NID>/,/<\/NODE>/p" | \
						${GREP} "<PATCHSET><ID>" | sed 's/[^0-9]*//g'`
	                do
	                    #NEW method#
	                   	${GREP} "^${PATCH_ID}/"  ${TYPE2_IN_VTREE} | awk -F"/" '{print $2}' >> ${PATCHES_ALL}
	                done	
	                
		    	elif [ ! "$DOWNLOAD_FROM_DATE" = "" ]; then
		    		echo "Downloading patches from $DOWNLOAD_FROM_DATE only"
		    		echo "Generating list of patches and timestamps..."
		    		${GREP} -A4 "<Advisory_id>" incidents_info.xml | awk '/<Advisory_id>/ {print d; d="";} { d=d" "$0} END { print d}' | awk '{print $1 $5}'| awk -F">" '{print $2 "<" $4}' |awk -F"<" '{print $3 ":" $1}'|${GREP} ':[0-9]\{6\}-[0-9]\{2\}' > /var/tmp/harvester-${CHANNEL}-timestamps.$$
		    		STARTING_DATE=`perl -e "use Time::Local; print timelocal(0,1,1,1,${DOWNLOAD_FROM_DATE_mm},${DOWNLOAD_FROM_DATE_YYYY});"`
		    		echo "Generating a patchlist..."
		    		for PATCH_LINE in `cat /var/tmp/harvester-${CHANNEL}-timestamps.$$`;do
		    			PATCH_TIMESTAMP=`echo "$PATCH_LINE" | cut -d':' -f1`
		    			if [ $PATCH_TIMESTAMP -ge $STARTING_DATE ]; then
		    				#echo "LINE:$PATCH_LINE   Patch_stamp:$PATCH_TIMESTAMP Start:$STARTING_DATE"
		    				PATCH_NAME=`echo "$PATCH_LINE" | cut -d':' -f2`
		    				echo $PATCH_NAME >> ${PATCHES_ALL}
		    			#else
		    				#echo "TOO OLD: LINE:$PATCH_LINE   Patch:$PATCH_TIMESTAMP Start:$STARTING_DATE"
		    			fi
		    		done
		    		rm -f /var/tmp/harvester-${CHANNEL}-timestamps.$$
		    	else
					# Search for all the patches in vtree for Solaris channels
					cat ${TYPE2_IN_VTREE} | awk -F"/" '{print $2}' > ${PATCHES_ALL}
	 			fi
	 			# Sort the list without revision number
			    echo "Sorting..."
			    sync
			    cat ${PATCHES_ALL} | awk -F"-" '{print $1}' | sort -u > ${PATCHES_SORTED}.tmp
			    mv ${PATCHES_SORTED}.tmp ${PATCHES_SORTED}
				
			    # Generate the list of patches to download
			    echo "Determining revision (last ${REVISIONS} revision(s) for each patch) ..."
			    for PATCH_ID in `cat ${PATCHES_SORTED}`; do
					${GREP} ${PATCH_ID} ${PATCHES_ALL} | tail -${REVISIONS} >> ${PATCHES_DOWNLOAD}
			    done
			    echo "Done"
	 		else		#### download patches from a file
	 			echo "Downloading patches from file ${PATCHES_FROM_FILE}"	
	 			if [ -f ${PATCHES_FROM_FILE} ]; then
	 				grep "CPU" ${PATCHES_FROM_FILE} >/dev/null 2>/dev/null
	 				if [ $? -eq 0 ]; then
	 					echo "WARNING: The Harvester does not support the download of CPU bundles as Ops Center does not deploy those directly (it is using XML instead - bug 17988717)."
	 					echo "         The CPU bundle will download but it will not work if you try to deploy it via Ops Center."
	 					echo ""
	 				fi
					cp -p ${PATCHES_FROM_FILE} ${PATCHES_DOWNLOAD}
				elif [ -f $scriptDir/${PATCHES_FROM_FILE} ]; then
					grep "CPU" ${PATCHES_FROM_FILE} >/dev/null 2>/dev/null
	 				if [ $? -eq 0 ]; then
	 					echo "WARNING: The Harvester does not support the download of CPU bundles as Ops Center does not deploy those directly (it is using XML instead - bug 17988717)."
	 					echo "         The CPU bundle will download but it will not work if you try to deploy it via Ops Center."
	 					echo ""
	 				fi
					cp -p $scriptDir/${PATCHES_FROM_FILE} ${PATCHES_DOWNLOAD}
				else
					echo "ERROR: Could not find ${PATCHES_FROM_FILE}, try using full path"
					echo ""
					exit
				fi
			fi
            
            # 
            # Begin loop to download patches
            # 
            PATCH_SUBDIR="all_unsigned"
            sync
            sleep 1
		    NO_OF_PATCHES=`wc -l ${PATCHES_DOWNLOAD} | awk '{print $1}'`
			echo "${NO_OF_PATCHES} patches will be downloaded"
			for PATCH_ID in `cat ${PATCHES_DOWNLOAD}`
		    do
				check_max_threads ""
				cd $scriptDir
				if [ "$PATCHES_FROM_FILE" = "" ]; then		#### Not downloading from file, show channel
					./download.sh "Patch ${PATCH_ID} for ${CHANNEL}" ${KB}/${PATCH_SUBDIR}/${PATCH_ID}.zip ${PATCHES_DIR}/ "" ${CONFIG_FILES}/${CHANNEL}/signatures-solaris.txt 2>&1 &
				else 
					./download.sh "Patch ${PATCH_ID}" ${KB}/${PATCH_SUBDIR}/${PATCH_ID}.zip ${PATCHES_DIR}/  2>&1 &
				fi
				let NO_OF_PATCHES=${NO_OF_PATCHES}-1
				let TEMP_VALUE=${NO_OF_PATCHES}%50
				if [ ${TEMP_VALUE} == 0 ] && [ ! "$NO_OF_PATCHES" = "0" ]; then
				    echo "About ${NO_OF_PATCHES} remaining to download..."
				fi
			done
		    rm -f ${PATCHES_ALL}
		    rm -f ${PATCHES_SORTED}
		    rm -f ${PATCHES_DOWNLOAD}
		    rm -f ${TYPE2_IN_VTREE}
            DOWNLOADED_PATCHES_FOR="${DOWNLOADED_PATCHES_FOR} ${CHANNEL}"
		fi		### end of "if  [ ${SOLARIS_CHECK} = 0 ] && [ ${CHANNEL_CHECK} = 0 ] && [ ${DOWNLOAD_PATCHES} = 1 ]; then"
		rm -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/vtree.xml >/dev/null 2>/dev/null
		rm -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/config_files/${CHANNEL}/incidents_info.xml >/dev/null 2>/dev/null
		
		########################################################################################
		############################# END OF DOWNLOAD SOLARIS PATCHES ##########################
		########################################################################################
	fi
done
#### End of: MODIFY PROXY.TGZ OF SOLARIS HERE ####

sync
sleep 1		### allow downloads to start

wait_for_threads_to_finish ""

sync
sleep 1

#### Some cleanup
rm -f $COOKIES_FILE	>/dev/null 2>/dev/null					### cleanup after downloading

####################################################################################################################
######## Scan all_unsigned to see what patches do we have, use that list to determine which are new patches ########
####################################################################################################################
NEW_COUNT=0
rm -rf /var/tmp/harvester-new-count.*
echo "$NEW_COUNT" > /var/tmp/harvester-new-count.$$
DATED_FOLDER=""
if [ "$MARK_NEW_PATCHES" = "1" ] && [ ${DOWNLOAD_PATCHES} = 1 ]; then			#### user requested to create hard linkes in a dated folder
	###### PATCHES HERE, RPMS LATER
	PATCHCOUNT=`ls ${PATCHES_DIR}/*.zip 2>/dev/null | wc -l`
	if [ $PATCHCOUNT -gt 0 ]; then
		### Only if we have any patches
		echo
		echo "Marking new patches: scanning which patches are new..."
		cd ${PATCHES_DIR}
		ls *.zip > /var/tmp/patchlist.txt.$$ 2>/dev/null
		mkdir -p ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/			### just in case we dont have it yet
		if [ -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/patchlist.txt ]; then
			########## need to compare current list to previous list #######
			cat /var/tmp/patchlist.txt.$$ | while read patch; do
				grep "^$patch" ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/patchlist.txt >/dev/null 2>/dev/null
				NEWPATCH=$?			##### not 0 if not there
				#### Also check if was redownloaded
				if [ -f ${STANDALONE_DIR}/all_unsigned/${patch}.NEWSIG ]; then
					#### This was just downloaded as a new signature, so it will be in the original patchlist so we won't know it needs to be marked
					SIGNATUREPATCH=1
				else
					SIGNATUREPATCH=0
				fi
				
				if [ $NEWPATCH -ne 0 ] || [ $SIGNATUREPATCH -eq 1 ];then		### if it's not there, or if there was a signature download
					#echo "$patch is NEW++++++++++++++++++++++++++"
					NEW_COUNT=`expr $NEW_COUNT + 1`
					echo "$NEW_COUNT" > /var/tmp/harvester-new-count.$$
					#echo "$patch" >> ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/patchlist.txt		#### add it to the main list
					#echo "$patch" >> ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/patchlist.txt.$$		#### add it to today's list
					if [ "$DATED_FOLDER" = "" ]; then		### If we didn't set folder name yet
						### Find first available name
						TODAYS=`date +"%Y-%b-%d"`
						if [ -d "${STANDALONE_DIR}/all_unsigned-${TODAYS}" ]; then		### if folder already exists
							DATE_COUNT=1
							while [ -d "${STANDALONE_DIR}/all_unsigned-${TODAYS}.${DATE_COUNT}" ]; do
								DATE_COUNT=`expr $DATE_COUNT + 1`			### keep increasing folder count until no folder exists
							done
							DATED_FOLDER="${STANDALONE_DIR}/all_unsigned-${TODAYS}.${DATE_COUNT}"
						else
							DATED_FOLDER="${STANDALONE_DIR}/all_unsigned-${TODAYS}"
						fi
						echo "Marking new patches with hardlinks in folder ${DATED_FOLDER}"
					fi
					##### use FOLDER_NAME
					mkdir -p $DATED_FOLDER
					### Create hardlink to real patch
					# We're currently inside the PATCHES_DIR (all_unsigned) folder
					ln $patch ${DATED_FOLDER}/${patch}
				#else 
					#echo "$patch is already in the list"
				fi
				rm -f /var/tmp/patchlist.txt.$$ >/dev/null 2>/dev/null
			done
			if [ -f /var/tmp/harvester-new-count.$$ ]; then
				NEW_COUNT=`cat /var/tmp/harvester-new-count.$$`
				rm -f /var/tmp/harvester-new-count.$$
			fi
			if [ ! "$NEW_COUNT" = "0" ]; then
				echo "Downloaded $NEW_COUNT new patches."
			else
				echo "No new patches were downloaded."
			fi
		fi
	fi
	###### RPMS HERE
	NEW_COUNT=0
	rm -rf /var/tmp/harvester-new-count.*
	echo "$NEW_COUNT" > /var/tmp/harvester-new-count.$$
	DATED_FOLDER=""
	RPMCOUNT=`find ${RPMS_DIR} -name "*.rpm" 2>/dev/null | wc -l`
	if [ $RPMCOUNT -gt 0 ]; then
		### Only if we have any RPMs
		echo
		echo "Marking new RPMs: scanning which RPMs are new..."
		cd ${RPMS_DIR}
		find . -name "*.rpm" > /var/tmp/rpmlist.txt.$$ 2>/dev/null
		mkdir -p ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/			### just in case we dont have it yet
		if [ -f ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/rpmlist.txt ]; then
			########## need to compare current list to previous list #######
			cat /var/tmp/rpmlist.txt.$$ | while read rpm; do
				grep "^${rpm}" ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/rpmlist.txt >/dev/null 2>/dev/null
				NEWRPM=$?			##### not 0 if not there
				#### Also check if was redownloaded
				if [ -f ${STANDALONE_DIR}/RPMs/${CHANNEL}/${rpm}.NEWSIG ]; then
					#### This was just downloaded as a new signature, so it will be in the original rpmlist so we won't know it needs to be marked
					SIGNATURERPM=1
				else
					SIGNATURERPM=0
				fi
				
				if [ $NEWRPM -ne 0 ] || [ $SIGNATURERPM -eq 1 ];then		### if it's not there, or if there was a signature download
					#echo "$rpm is NEW++++++++++++++++++++++++++"
					NEW_COUNT=`expr $NEW_COUNT + 1`
					echo "$NEW_COUNT" > /var/tmp/harvester-new-count.$$
					#echo "$rpm" >> ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/rpmlist.txt		#### add it to the main list
					#echo "$rpm" >> ${STANDALONE_DIR}/${EXCLUDE_FOLDER}/rpmlist.txt.$$		#### add it to today's list
					if [ "$DATED_FOLDER" = "" ]; then		### If we didn't set folder name yet
						### Find first available name
						TODAYS=`date +"%Y-%b-%d"`
						if [ -d "${STANDALONE_DIR}/RPMs-${TODAYS}" ]; then		### if folder already exists
							DATE_COUNT=1
							while [ -d "${STANDALONE_DIR}/RPMs-${TODAYS}.${DATE_COUNT}" ]; do
								DATE_COUNT=`expr $DATE_COUNT + 1`			### keep increasing folder count until no folder exists
							done
							DATED_FOLDER="${STANDALONE_DIR}/RPMs-${TODAYS}.${DATE_COUNT}"
						else
							DATED_FOLDER="${STANDALONE_DIR}/RPMs-${TODAYS}"
						fi
						echo "Marking new RPMs with hardlinks in folder ${DATED_FOLDER}"
					fi
					##### use FOLDER_NAME
					mkdir -p $DATED_FOLDER
					TARGETFOLDER=`dirname "$rpm"`		### for example, this will contain "./OEL_6_0_AMD64"
					mkdir -p ${DATED_FOLDER}/${TARGETFOLDER}
					### Create hardlink to real rpm
					# We're currently inside the RPMS_DIR (RPMs)/CHANNEL folder
					ln $rpm ${DATED_FOLDER}/${rpm}
				#else 
					#echo "$rpm is already in the list"
				fi
				rm -f /var/tmp/rpmlist.txt.$$ >/dev/null 2>/dev/null
			done
			if [ -f /var/tmp/harvester-new-count.$$ ]; then
				NEW_COUNT=`cat /var/tmp/harvester-new-count.$$`
				rm -f /var/tmp/harvester-new-count.$$
			fi
			if [ ! "$NEW_COUNT" = "0" ]; then
				echo "Downloaded $NEW_COUNT new RPMs."
			else
				echo "No new RPMs were downloaded."
			fi
		fi
	fi
fi

#### Clean markers for new signatures (these are used for "MARK NEW PATCHES")
find ${STANDALONE_DIR} -name "*.NEWSIG" | while read FILETOREMOVE; do
	if [ -f $FILETOREMOVE ]; then
		rm -f $FILETOREMOVE
	fi
done

sync
sleep 1

###### Copy script to determine destination folder
if [ -d ${STANDALONE_DIR}/all_unsigned/ ]; then
	cp -p ${scriptDir}/copy_patches_to_oc.sh ${STANDALONE_DIR}/all_unsigned/ 
fi
### Find folders with stamp
TODAYS=`date +"%Y-%b-%d"`
if [ -d "${STANDALONE_DIR}/all_unsigned-${TODAYS}" ]; then		### if folder already exists
	cp -p ${scriptDir}/copy_patches_to_oc.sh ${STANDALONE_DIR}/all_unsigned-${TODAYS}/
	DATE_COUNT=1
	while [ -d "${STANDALONE_DIR}/all_unsigned-${TODAYS}.${DATE_COUNT}" ]; do
		cp -p ${scriptDir}/copy_patches_to_oc.sh ${STANDALONE_DIR}/all_unsigned-${TODAYS}.${DATE_COUNT}/
		DATE_COUNT=`expr $DATE_COUNT + 1`			### keep increasing folder count until no folder exists
	done
fi


if [ "$PATCHES_FROM_FILE" = "" ] && [ "$RPMS_FROM_FILE" = "" ]; then		### if not downloading patches from a file
	echo
	echo "Creating tarball..."
	
	EXCLUDE_FILE=/var/tmp/exclude_file.$$
	rm -f ${EXCLUDE_FILE}
	echo "*.OK" > ${EXCLUDE_FILE}
	echo $EXCLUDE_FOLDER >> ${EXCLUDE_FILE}
	echo "RPMs" >> ${EXCLUDE_FILE}
	echo "all_unsigned" >>  ${EXCLUDE_FILE}
	echo "standalone-*" >>  ${EXCLUDE_FILE}
	echo "all_unsigned*" >>  ${EXCLUDE_FILE}
	#echo "proxy.tgz.orig" >> ${EXCLUDE_FILE}
	cd ${STANDALONE_DIR}
	chmod -R a+r *
	chmod -R a+x *
	sync
	sleep 2
	${GTAR} zcf ${STANDALONE_DIR}/standalone-${TAR_DATE}.tar.gz * --exclude-from ${EXCLUDE_FILE}
	result=$?
	rm -f ${EXCLUDE_FILE}
else
	result=0
fi

if [ ${result} -ne 0 ];then
    echo "ERROR.  There was an error creating the tarball. Check ${WGET_LOG} for more information"
    exit 1
else

    # generate a little summary of what we did and display it
    # for the user.
    #

    END_TIME=`date`

	echo
	if [ "$PATCHES_FROM_FILE" = "" ] && [ "$RPMS_FROM_FILE" = "" ]; then		### if not downloading patches from a file
	    echo "Tarball successfully created at ${STANDALONE_DIR}/standalone-${TAR_DATE}.tar.gz"
	    echo
	    REPORT_CHANNELS_LIST=`echo ${CHANNELS_LIST} | tr '\n' ' '`
	    REPORT_CHANNELS_COUNT=`echo ${REPORT_CHANNELS_LIST} | wc -w | awk '{print $1}'`
	    echo "Tarball includes KB (Metadata) for ${REPORT_CHANNELS_COUNT} distributions."
	    echo
	fi
    echo "Start time:        ${START_TIME}"
    echo "Completion time:   ${END_TIME}"
    echo 
    print_instructions $TAR_DATE
	exit 0
fi
